﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Data;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.Migration.Toolkit;
using Microsoft.TeamFoundation.Migration.Toolkit.VC;
using System.Xml;
using System.IO;
using Tests.Framework;
using System.Diagnostics;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.VersionControl.ChangeGrouping and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.VersionControl.ChangeGrouping Unit Tests
    ///</summary>
    [TestClass()]
    public class ChangeGroupingTest : MigrationTestCaseBase
    {
        VersionControlSession session;

        #region Additional test attributes

        [TestInitialize()]
        public override void Initialize()
        {
            base.Initialize();

            ConfigFile.WriteAndLoad(ConfigParameters);
            DataAccessManager.Current.CreateSchema(true);
            session = new VersionControlSession();
            session.Id = TestContext.TestName;
        }

        [TestCleanup]
        public override void Cleanup()
        {
            base.Cleanup();

            string serverName = TestEnvironment.SqlServer;
            SqlUtil.DropDB(serverName, ConfigParameters.SqlInfo.Database);

            session = null;
        }

        private void createExtraGroup(ChangeGroupingMananger manager)
        {
            ChangeGrouping extraGroup = CreateChangeGroup(manager, "extra");
            extraGroup.Status = ChangeStatus.Skipped;
            extraGroup.Save();
        }
        private ChangeStatus QueryStatus(string name)
        {
            string query = string.Format("Select status from ChangeGroups where Name='{0}'", name);

            return (ChangeStatus)SqlUtil.ExecSqlQuery(query, string.Format("server={0};database={1};Integrated Security=SSPI", TestEnvironment.SqlServer, TestContext.TestName));
        }
        #endregion


        /// <summary>
        ///A test for Next ()
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.VersionControl.dll")]
        [TestMethod()]
        public void GetNextTest()
        {
            ChangeGroupingMananger s2t = new SqlChangeGroupingMananger(
                null, null, session, MigrationDirection.SourceToTfs);

            ChangeGroupingMananger t2s = new SqlChangeGroupingMananger(
                null, null, session, MigrationDirection.TfsToSource);


            ChangeGrouping group1 = s2t.Create(string.Empty);
            group1.ChangeTimeUtc = DateTime.Now;
            group1.Comment = "Comment for group 1";
            group1.Owner = "Any Username";
            group1.Status = ChangeStatus.Pending;
            group1.Save();
            Assert.IsTrue(group1.ChangeGroupId != int.MinValue); // min is default

            ChangeGrouping group2 = s2t.Next();
            Assert.IsNotNull(group2);
            Assert.AreEqual<DateTime>(group1.ChangeTimeUtc, group2.ChangeTimeUtc);
            Assert.AreEqual(group1.Comment, group2.Comment);
            Assert.AreEqual(group1.ExecutionOrder, group2.ExecutionOrder);
            Assert.AreEqual(group1.Status, group2.Status);
            Assert.AreEqual(group1.Owner, group2.Owner);
            Assert.AreEqual(group1.Direction, group2.Direction);
            Assert.AreEqual(group1.ChangeGroupId, group2.ChangeGroupId);
            Assert.AreEqual(group1.Direction, MigrationDirection.SourceToTfs);

            group2.Status = ChangeStatus.InProgress;
            group2.Save();

            ChangeGrouping shouldBeNull = s2t.Next();
            Assert.IsNull(shouldBeNull);

            group1.Status = ChangeStatus.Complete;
            group1.Save();

            shouldBeNull = s2t.Next();
            Assert.IsNull(shouldBeNull);

            ChangeGrouping next = null;
            long[] orders = new long[10];
            for (int i = 0; i < orders.Length; i++)
            {
                next = t2s.Create(string.Empty);
                next.ChangeTimeUtc = DateTime.Now;
                next.Comment = "Comment for group 1";
                next.Owner = "Any Username";
                next.Status = ChangeStatus.Pending;
                next.Direction = MigrationDirection.TfsToSource;
                next.Save();
                Assert.IsTrue(next.ChangeGroupId != int.MinValue); // min is default
                orders[i] = next.ExecutionOrder;
            }

            next = null;
            long order = 0;
            while ((next = t2s.Next()) != null)
            {
                Assert.AreEqual<long>(orders[order++], next.ExecutionOrder);
                next.Status = ChangeStatus.Complete;
                next.Save();
            }

            Assert.AreEqual<long>(order, orders.Length);

            shouldBeNull = t2s.Next();
            Assert.IsNull(shouldBeNull);
        }

        /// <summary>
        ///A test for CreateAction ()
        ///</summary>
        [TestMethod()]
        public void ChangeGroupingValidator()
        {
            ChangeGroupingMananger s2t = new SqlChangeGroupingMananger(
                null, null, session, MigrationDirection.TfsToSource);

            ChangeGrouping group = new SqlChangeGrouping(s2t);

            string comment = "this is a non null comment";
            group.Comment = comment;
            Assert.AreEqual<string>(group.Comment, comment);

            // a null comment should become an empty comment
            group.Comment = null;
            Assert.AreEqual<string>(group.Comment, string.Empty);

            try
            {
                group.Status = (ChangeStatus)255;
                Assert.Fail("Setting status to bogus value should have thrown a MigrationException");
            }
            catch (MigrationException)
            {
            }
        }

        /*
        /// <summary>
        ///A test for CreateAction ()
        ///</summary>
        [TestMethod()]
        public void CreateActionTest()
        {
            ChangeGrouping group1 = new ChangeGrouping(null);
            group1.ChangeTime = DateTime.Now;
            group1.Comment = "Comment for group 1";
            group1.Owner = "Any Username";
            group1.Status = ChangeStatus.Pending;

            for (int i = 0; i < 5; i++)
            {
                IMigrationAction action = group1.CreateAction();
                action.TFSTargetItem = new ItemSpec(string.Format("$/TeamProject/TargetItem{0}", i), RecursionType.None, i);
                action.TFSSourceItem = new ItemSpec(string.Format("$/TeamProject/SourceItem{0}", i), RecursionType.None, i);
                action.SourceItem = new SourceItem<int>(string.Format("SourceItemName{0}", i), i);
                action.Action = ChangeType.Delete;
                action.Recursive = ((i % 2) == 0);
                action.ResolveType = ResolveMethod.Theirs;
            }

            group1.Save();

            ChangeGrouping sameGroup = ChangeGrouping.Next(null);

            Assert.IsNotNull(sameGroup);
            IMigrationAction[] actionItems = sameGroup.Actions;

            Assert.AreEqual<int>(actionItems.Length, 5);

            for (int i = 0; i < actionItems.Length; i++)
            {
                Assert.AreEqual<string>(actionItems[i].TFSTargetItem.Target, string.Format("$/TeamProject/TargetItem{0}", i));
                Assert.AreEqual<int>(actionItems[i].TFSTargetItem.DeletionId, i);
                Assert.AreEqual<RecursionType>(actionItems[i].TFSTargetItem.RecursionType, RecursionType.None);

                Assert.AreEqual<string>(actionItems[i].TFSSourceItem.Target, string.Format("$/TeamProject/SourceItem{0}", i));
                Assert.AreEqual<int>(actionItems[i].TFSSourceItem.DeletionId, i);
                Assert.AreEqual<RecursionType>(actionItems[i].TFSSourceItem.RecursionType, RecursionType.None);

                Assert.AreEqual<string>(actionItems[i].SourceItem.Name, string.Format("SourceItemName{0}", i));
                Assert.AreEqual<int>(actionItems[i].SourceItem.Version, i);

                Assert.AreEqual<ChangeType>(actionItems[i].Action, ChangeType.Delete);
                Assert.AreEqual<bool>(actionItems[i].Recursive, ((i % 2) == 0));

                Assert.AreEqual<int>(actionItems[i].ChangeGroup.ChangeGroupId, sameGroup.ChangeGroupId);

                Assert.AreEqual<ResolveMethod>(actionItems[i].ResolveType, ResolveMethod.Theirs);
            }
        }
         * */

        /// <summary>
        ///A test for PromoteAnalysisToPending (Guid)
        ///</summary>
        [TestMethod()]
        public void PromoteAnalysisToPendingTest()
        {
            string sessionId = TestContext.TestName;

            ChangeGroupingMananger t2sStub = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.TfsToSource);
            ChangeGroupingMananger s2tStub = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group = s2tStub.Create(string.Empty);
            group.SessionId = sessionId;
            group.Owner = "testuser";
            group.Status = ChangeStatus.Analysis;
            group.ChangeTimeUtc = DateTime.Now;
            group.Comment = string.Empty;

            group.Save();

            ChangeGrouping group2 = t2sStub.Create(string.Empty);
            group2.SessionId = sessionId;
            group2.Owner = "testuser";
            group2.Status = ChangeStatus.Analysis;
            group2.ChangeTimeUtc = DateTime.Now;
            group2.Comment = string.Empty;

            group2.Save();
           
            int actual1 = s2tStub.PromoteAnalysisToPending();
            Assert.AreEqual<int>(1, actual1);

            int actual2 = t2sStub.PromoteAnalysisToPending();
            Assert.AreEqual<int>(1, actual2);


            group.Status = ChangeStatus.Skipped;
            group.Save();

            group2.Status = ChangeStatus.Skipped;
            group2.Save();
        }

        ///<summary>
        ///Scenario: Create a ChangeGrouping. Call PromoteAnalysisToPending before calling Save
        ///Expected Result: No-Op
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Create a ChangeGrouping. Call PromoteAnalysisToPending before calling Save")]
        public void PromoteBeforeSaveTest()
        {
            ChangeGroupingMananger sourceToTfsManager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping sourceToTfsGroup = CreateChangeGroup(sourceToTfsManager);

            sourceToTfsManager.PromoteAnalysisToPending();

            Assert.AreEqual(ChangeStatus.Analysis, sourceToTfsGroup.Status);
        }

        ///<summary>
        ///Scenario: Call Next with All change groupings in Analysis state
        ///Expected Result: Returns Null
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Call Next with All change groupings in Analysis state")]
        public void NextOnAnalysisTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group = CreateChangeGroup(manager);
            group.Save();

            Assert.IsNull(manager.Next());
        }

        ///<summary>
        ///Scenario: Call Next with All change Groupings in Skipped state
        ///Expected Result: Returns Null
        ///</summary>
        [TestMethod(), Priority(2), Owner("curtisp")]
        [Description("Call Next with All change Groupings in Skipped state")]
        public void NextOnSkippedTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group = CreateChangeGroup(manager);
           
            group.Status = ChangeStatus.Skipped;
            group.Save();

            Assert.IsNull(manager.Next());
        }

        ///<summary>
        ///Scenario: Call Promote when multiple change groups are in Analysis state
        ///Expected Result: All ChangeGroups in Analysis state are put in Pending state. No other changes.
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Call Promote when multiple change groups are in Analysis state")]
        public void PromoteMultipleChangeGroupsTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager,"group1");
            ChangeGrouping group2 = CreateChangeGroup(manager, "group2");
            createExtraGroup(manager);
            group1.Save();
            group2.Save();

            manager.PromoteAnalysisToPending();

            Assert.AreEqual(ChangeStatus.Pending,QueryStatus("group1"));
            Assert.AreEqual(ChangeStatus.Pending, QueryStatus("group2"));
            Assert.AreEqual(ChangeStatus.Skipped, QueryStatus("extra"));  
        }

        ///<summary>
        ///Scenario: Call Demote when multiple change groups are in InProgress state
        ///Expected Result: All ChangeGroups in InProgress state are put in Pending state. No other changes
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Call Demote when multiple change groups are in InProgress state")]
        public void DemoteMultipleChangeGroupsTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            ChangeGrouping group2 = CreateChangeGroup(manager, "group2");
            createExtraGroup(manager);
            group1.Status = ChangeStatus.InProgress;
            group2.Status = ChangeStatus.InProgress;
            group1.Save();
            group2.Save();

            manager.DemoteInProgressActionsToPending();

            Assert.AreEqual(ChangeStatus.Pending, QueryStatus("group1"));
            Assert.AreEqual(ChangeStatus.Pending, QueryStatus("group2"));
            Assert.AreEqual(ChangeStatus.Skipped, QueryStatus("extra"));  
        }

        ///<summary>
        ///Scenario: Call Demote when only one ChangeGroup is in InProgress state
        ///Expected Result: One ChangeGroup is demoted to Pending. No other changes.
        ///</summary>
        [TestMethod(), Priority(2), Owner("curtisp")]
        [Description("Call Demote when only one ChangeGroup is in InProgress state")]
        public void DemoteSingleChangeGroupTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            createExtraGroup(manager);
            group1.Status = ChangeStatus.InProgress;
            group1.Save();

            manager.DemoteInProgressActionsToPending();

            Assert.AreEqual(ChangeStatus.Pending, QueryStatus("group1"));
            Assert.AreEqual(ChangeStatus.Skipped, QueryStatus("extra"));  
        }

        ///<summary>
        ///Scenario: Call Save on a condition that makes the CLR throw a SQLException
        ///Expected Result: SQLException bubbles up
        ///</summary>
        [TestMethod(), Priority(2), Owner("curtisp")]
        [Description("Call Save on a condition that makes the CLR throw a SQLException")]
        [ExpectedException(typeof(SqlException))]
        public void SaveSQLExceptionTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            SqlUtil.DropDB(TestEnvironment.SqlServer, TestContext.TestName);

            group1.Save();
        }

        ///<summary>
        ///Scenario: Call Promote on a single session when there are Analysis changes in 2 sessions.
        ///Expected Result: Only the changes in Session1 are promoted
        ///</summary>
        [TestMethod(), Priority(2), Owner("curtisp")]
        [Description("Call Promote on a single session when there are Analysis changes in 2 sessions.")]
        public void PromoteMultipleSessionsTest()
        {
            ChangeGroupingMananger session1Manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            VersionControlSession session2 = new VersionControlSession();
            session2.Id = "Session2";
            ChangeGroupingMananger session2Manager = new SqlChangeGroupingMananger(null, null, session2, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(session1Manager, "group1");
            ChangeGrouping group2 = CreateChangeGroup(session2Manager, "group2");

            group1.Save();
            group2.Save();

            session1Manager.PromoteAnalysisToPending();

            Assert.AreEqual(ChangeStatus.Pending, QueryStatus("group1"));
            Assert.AreEqual(ChangeStatus.Analysis, QueryStatus("group2"));
        }

        ///<summary>
        ///Scenario: Call Demote on a single session when there are InProgress changes in 2 sessions.
        ///Expected Result: Only the changes in Session1 are demoted
        ///</summary>
        [TestMethod(), Priority(2), Owner("curtisp")]
        [Description("Call Demote on a single session when there are InProgress changes in 2 sessions.")]
        public void DemoteMultipleSessionsTest()
        {
            ChangeGroupingMananger session1Manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            VersionControlSession session2 = new VersionControlSession();
            session2.Id = "Session2";
            ChangeGroupingMananger session2Manager = new SqlChangeGroupingMananger(null, null, session2, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(session1Manager, "group1");
            ChangeGrouping group2 = CreateChangeGroup(session2Manager, "group2");
            group1.Status= ChangeStatus.InProgress;
            group2.Status = ChangeStatus.InProgress;

            group1.Save();
            group2.Save();

            session1Manager.DemoteInProgressActionsToPending();

            Assert.AreEqual(ChangeStatus.Pending, QueryStatus("group1"));
            Assert.AreEqual(ChangeStatus.InProgress, QueryStatus("group2"));
        }


        ///<summary>
        ///Scenario: Attempt the set the public properties/methods after locking the change grouping. 
        ///Expected Result: All properties/methods but Status throw a MigrationException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Attempt the set the public properties/methods after locking the change grouping. ")]
        public void ChangesAfterLockTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            Microsoft_TeamFoundation_Migration_Toolkit_VC_ChangeGroupingAccessor group1Acessor = new Microsoft_TeamFoundation_Migration_Toolkit_VC_ChangeGroupingAccessor(group1);
            group1Acessor.Locked = true;
            try
            {
                group1.Name = "";
                Assert.Fail("Expected a Migration Exception when trying to edit a changegroup after lock");
            }
            catch (MigrationException){}

            try
            {
                group1.ChangeTimeUtc = DateTime.Now;
                Assert.Fail("Expected a Migration Exception when trying to edit a changegroup after lock");
            }
            catch (MigrationException) { }

            try
            {
                group1.Comment = "";
                Assert.Fail("Expected a Migration Exception when trying to edit a changegroup after lock");
            }
            catch (MigrationException) { }

            try
            {
                group1.Direction = MigrationDirection.Unknown;
                Assert.Fail("Expected a Migration Exception when trying to edit a changegroup after lock");
            }
            catch (MigrationException) { }

            try
            {
                group1.ExecutionOrder = 0;
                Assert.Fail("Expected a Migration Exception when trying to edit a changegroup after lock");
            }
            catch (MigrationException) { }

            try
            {
                group1.Owner = "";
                Assert.Fail("Expected a Migration Exception when trying to edit a changegroup after lock");
            }
            catch (MigrationException) { }

            try
            {
                group1.SessionId = "";
                Assert.Fail("Expected a Migration Exception when trying to edit a changegroup after lock");
            }
            catch (MigrationException) { }

        }

        ///<summary>
        ///Scenario: Migrate a change with Owner variable set to null
        ///Expected Result: Null Is allowed
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Migrate a change with Owner variable set to null")]
        public void NullOwnerTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            group1.Owner = null;

            group1.Save();

            Assert.IsNull(group1.Owner);
        }

        ///<summary>
        ///Scenario: Migrate a change with Owner variable set to empty string
        ///Expected Result: Empty String is Allowed
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Migrate a change with Owner variable set to empty string")]
        public void EmptyOwnerTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            group1.Owner = string.Empty;

            group1.Save();

            Assert.AreEqual(string.Empty, group1.Owner);
        }

        ///<summary>
        ///Scenario: Migrate a change with Comment variable set to empty string
        ///Expected Result: The comment is empty string
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Migrate a change with Comment variable set to empty string")]
        public void EmptyCommentTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            group1.Comment = string.Empty;

            group1.Save();

            Assert.AreEqual(string.Empty,group1.Comment);
        }

        ///<summary>
        ///Scenario: Migrate a change with SessionId variable set to null
        ///Expected Result: Defaults to Empty string
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Migrate a change with SessionId variable set to null")]
        public void NullSessionIdTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            group1.SessionId = null;

            group1.Save();

            Assert.AreEqual(string.Empty, group1.Comment);
        }

        ///<summary>
        ///Scenario: Migrate a change with SessionId variable set to empty string
        ///Expected Result: Empty string is used
        ///</summary>
        [TestMethod(), Priority(2), Owner("curtisp")]
        [Description("Migrate a change with SessionId variable set to empty string")]
        public void EmptySessionIdTestTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            group1.SessionId = string.Empty;

            group1.Save();

            Assert.AreEqual(string.Empty, group1.Comment);
        }

        ///<summary>
        ///Scenario: Call AddAction with a null action
        ///Expected Result: InvalidArgumentException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call AddAction with a null action")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AddNullActionTest()
        {
            ChangeGroupingMananger manager = new SqlChangeGroupingMananger(null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping group1 = CreateChangeGroup(manager, "group1");
            group1.AddAction(null);
        }

    }
}
