﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Text;
using System.Collections.Generic;
using Microsoft.TeamFoundation.Migration.Toolkit;
using System.Xml;
using System.Xml.Schema;
using Tests.Framework;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.Toolkit.SettingsSectionTest and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.Toolkit.SettingsSectionTest Unit Tests
    ///</summary>
    [TestClass()]
    public class SettingsSectionTest : ConfigManagerTestCaseBase
    {

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        #region AddSetting Method tests

        [TestMethod()]
        public void SettingsSection_AddSetting()
        {
            using (Stream configFile = this.GetType().Assembly.GetManifestResourceStream("Tests.ConfigurationManager.Test1.xml"))
            {
                using (MigrationConfiguration config = MigrationConfiguration.Load(configFile))
                {
                    foreach (Setting s in config.Settings)
                    {
                        Assert.IsNotNull(s.Name);
                        Assert.IsNotNull(s.Value);
                    }

                    config.AddSetting("add1", "added one value");
                    config.AddSetting(new Setting("add2", "added two value"));

                    Assert.AreEqual<string>(config.GetValue("add1"), "added one value");
                    Assert.AreEqual<string>(config.GetValue("add2"), "added two value");

                    DateTime oneHourAgo = DateTime.Now.Subtract(new TimeSpan(1, 0, 0));

                    config.AddSetting("onehourago", oneHourAgo.ToString());
                    DateTime oneHourAgo2 = config.GetValue<DateTime>("onehourago", DateTime.Now);

                    Assert.AreEqual<int>(oneHourAgo.Hour, oneHourAgo2.Hour);
                    Assert.AreEqual<int>(oneHourAgo.Second, oneHourAgo2.Second);

                    int notfound = config.GetValue<int>("notfound", 12345);

                    Assert.AreEqual<int>(notfound, 12345);

                    try
                    {
                        config.GetValue("notfound");
                        Assert.Fail("Not found value should have thrown a migration exception");
                    }
                    catch (MigrationException)
                    {
                    }
                }
            }
        }

        [TestMethod()]
        public void SettingsSection_AddSettingErrors()
        {
            using (Stream configFile = this.GetType().Assembly.GetManifestResourceStream("Tests.ConfigurationManager.Test1.xml"))
            {
                using (MigrationConfiguration config = MigrationConfiguration.Load(configFile))
                {
                    try
                    {
                        config.AddSetting(null);
                        Assert.Fail("Adding null setting should throw ArgumentNullException");
                    }
                    catch (ArgumentNullException)
                    {
                    }

                    config.AddSetting("foo", "bar");

                    try
                    {
                        config.AddSetting("foo", "bar");
                        Assert.Fail("Adding duplicate named setting should throw exception");
                    }
                    catch (MigrationException)
                    {
                    }

                    Setting s = new Setting("foo", "bar");

                    try
                    {
                        config.AddSetting(s);
                        Assert.Fail("Adding duplicate named setting should throw exception");
                    }
                    catch (MigrationException)
                    {
                    }

                }
            }
        }

        //Create a Settings Object explicitly using constructor.    Create the same Setting Object implicitly by calling 
        [TestMethod()]
        public void SettingsEqualityTest()
        {
            configFile.WriteAndLoad();

            String name = "SettingName";
            String value = "SettingValue";
            Setting setting = new Setting(name, value);


            MigrationConfiguration.Current.AddSetting(name, value);

            Assert.AreEqual(setting.Name, MigrationConfiguration.Current.Settings[MigrationConfiguration.Current.Settings.Length - 1].Name);
            Assert.AreEqual(setting.Value, MigrationConfiguration.Current.Settings[MigrationConfiguration.Current.Settings.Length - 1].Value);

        }

        //Valid Name Null Value
        [TestMethod()]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullValueAddSettingTest()
        {
            configFile.WriteAndLoad();

            String name = "SettingName";
            String value = null;

            MigrationConfiguration.Current.AddSetting(name, value);
        }

        //Valid Name; Integer Value expected, string value given. 
        [TestMethod()]
        public void WrongTypeValueSettingTest()
        {
            configFile.WriteAndLoad();

            String name = "SettingName";
            String value = "SettingValue";

            MigrationConfiguration.Current.AddSetting(name, value);
            int defaultValue = 1337;
            Assert.AreEqual(defaultValue, MigrationConfiguration.Current.GetValue<int>(name, defaultValue));

            int outValue;
            Assert.IsFalse(MigrationConfiguration.Current.TryGetValue<int>(name, out outValue));
            Assert.AreEqual(0, outValue);
        }

        //Valid Name Date time (mm/dd/yyyy) Value expected string mm/dd/yy given
        [TestMethod()]
        public void WrongFormatValueSettingTest()
        {
            configFile.WriteAndLoad();

            String name = "SettingName";
            //An Invalid date
            String value = "05/15/20006";

            MigrationConfiguration.Current.AddSetting(name, value);
            DateTime defaultValue = DateTime.Today;

            Assert.AreEqual(defaultValue, MigrationConfiguration.Current.GetValue<DateTime>(name, defaultValue));

            DateTime outValue;
            Assert.IsFalse(MigrationConfiguration.Current.TryGetValue<DateTime>(name, out outValue));
            //Default Value of DateTime
            Assert.AreEqual(DateTime.MinValue, outValue);
        }

        //Null Name Valid Value
        [TestMethod()]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullNameAddSettingsTest()
        {
            configFile.WriteAndLoad();

            String value = "SettingValue";

            MigrationConfiguration.Current.AddSetting(null, value);
        }
        #endregion AddSetting Tests

        #region Settings Config file tests
        //Valid Name and value Strings
        [TestMethod()]
        public void MainlineSettingsFileTest()
        {
            configFile.WriteAndLoad();


            validateSettings(MigrationConfiguration.Current, doc.SelectSingleNode("/Migration/Settings").ChildNodes);
            validateSettings(MigrationConfiguration.Current.VC, doc.SelectSingleNode("/Migration/VC/Settings").ChildNodes);
            validateSettings(MigrationConfiguration.Current.Wit, doc.SelectSingleNode("/Migration/WIT/Settings").ChildNodes);

            int loopIndex = 0;
            foreach (VersionControlSession sessionSettings in MigrationConfiguration.Current.VC.Sessions.Values)
            {
                XmlNodeList sessionSettingsNodes =
                    doc.SelectSingleNode("/Migration/VC/Sessions").ChildNodes[loopIndex].SelectSingleNode("Settings").ChildNodes;

                validateSettings(sessionSettings, sessionSettingsNodes);
                loopIndex++;
            }

            //WorkItemTrackingSessions Are stored in a dictionary which we cannot access via index
            //so must use for each loop and track index seperatly
            loopIndex = 0;
            foreach (WorkItemTrackingSession sessionSettings in MigrationConfiguration.Current.Wit.Sessions.Values)
            {
                XmlNodeList sessionSettingsNodes = 
                    doc.SelectSingleNode("/Migration/WIT/Sessions").ChildNodes[loopIndex].SelectSingleNode("Settings").ChildNodes;

                validateSettings(sessionSettings, sessionSettingsNodes);

                loopIndex++;
            }

        }

        private static void validateSettings(ISettingsSection settingsSection, XmlNodeList settingsNodes)
        {
            for (int i = 0; i < settingsSection.Settings.Length; i++)
            {
                Assert.AreEqual(settingsNodes[i].Attributes["name"].InnerText, settingsSection.Settings[i].Name);
                Assert.AreEqual(settingsNodes[i].Attributes["value"].InnerText, settingsSection.Settings[i].Value);

            }
        }

        //Valid Name Null Value
        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void NullValueSettingFileTest()
        {

 

            XmlNode node= doc.SelectSingleNode("/Migration/Settings/Setting");

            node.Attributes.Remove(node.Attributes["value"]);

            configFile.WriteAndLoad();
        }

        //Null Name Valid Value
        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void NullNameSettingFileTest()
        {


            XmlNode node = doc.SelectSingleNode("/Migration/Settings/Setting");

            node.Attributes.Remove(node.Attributes["name"]);

            configFile.WriteAndLoad();
           
        }

        //Valid Name Empty Value	
        [TestMethod()]
        public void EmptyValueSettingFileTest()
        {


            XmlNode node = doc.SelectSingleNode("/Migration/Settings/Setting");

            node.Attributes["value"].RemoveAll();

            configFile.WriteAndLoad();

            Assert.AreEqual("", MigrationConfiguration.Current.GetValue(node.Attributes["name"].InnerText));

        }

        //A new Settings Object with the same name value pair as a pervious setting  of the same scope	
        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void DuplicateSettingFileTest()
        {


            XmlNode node = doc.SelectSingleNode("/Migration/Settings/Setting");

            node.Attributes["name"].InnerText = node.NextSibling.Attributes["name"].InnerText;
            node.Attributes["value"].InnerText= node.NextSibling.Attributes["value"].InnerText;

            configFile.WriteAndLoad();
        }
        #endregion Settings Config File tests
    }

}
