﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.TeamFoundation.Migration.Toolkit;
using System.IO;
using System.Xml;
using System.Xml.Schema;
using Tests.Framework;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.Toolkit.VersionControlMapping and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.Toolkit.VersionControlMapping Unit Tests
    ///</summary>
    [TestClass()]
    public class VersionControlMappingTest : ConfigManagerTestCaseBase
    {

        #region Helper methods

        private static void navigateToMappingNode(XmlDocument doc, out XmlNode sessionNode, out XmlNode mappingNode)
        {

            sessionNode = doc.SelectSingleNode("/Migration/VC/Sessions/Session");
            //Mapping
            mappingNode = doc.SelectSingleNode("/Migration/VC/Sessions/Session/Mappings/Mapping"); ;

        }

        public static string InvalidVCPath
        {
            get
            {
                return @"$/Foo/$Bar/Test";
            }
        }


        #endregion

        #region Test Methods
        [TestMethod()]
        public void NoCloakTest()
        {


            XmlNode sessionNode;
            XmlNode mappingNode;
            navigateToMappingNode(doc, out sessionNode, out mappingNode);

            mappingNode.Attributes.Remove(mappingNode.Attributes["cloak"]);

            configFile.WriteAndLoad();
            VersionControlSession session = MigrationConfiguration.Current.VC.Sessions["Session0"];

            validateSessionAttributes(session, sessionNode);

            //validate Mappings
            Assert.AreEqual(mappingNode.Attributes["src"].InnerText, session.Mappings[0].Source);
            Assert.AreEqual(mappingNode.Attributes["tgt"].InnerText, session.Mappings[0].Target);
            Assert.AreEqual(false, session.Mappings[0].Cloak);

            //validate that methods can be called
            session.SynchronizeFull();
        }

        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void EmptyTargetTest()
        {

            XmlNode sessionNode;
            XmlNode mappingNode;
            navigateToMappingNode(doc, out sessionNode, out mappingNode);

            mappingNode.Attributes.Remove(mappingNode.Attributes["tgt"]);

            configFile.WriteAndLoad();

        }

        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void EmptySourceTest()
        {

            XmlNode sessionNode;
            XmlNode mappingNode;
            navigateToMappingNode(doc, out sessionNode, out mappingNode);

            mappingNode.Attributes.Remove(mappingNode.Attributes["src"]);
            configFile.WriteAndLoad();
        }


        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void ConflictingCloakTest()
        {


            XmlNode sessionNode;
            XmlNode mappingNode;
            navigateToMappingNode(doc, out sessionNode, out mappingNode);

            XmlNode sibblingNode = mappingNode.NextSibling;

            //copy source
            sibblingNode.Attributes["src"].Value = mappingNode.Attributes["src"].Value;
            //copy target
            sibblingNode.Attributes["tgt"].Value = mappingNode.Attributes["tgt"].Value;
            //set cloak
            mappingNode.Attributes["cloak"].Value = "true";
            sibblingNode.Attributes["cloak"].Value = "false";

            configFile.WriteAndLoad();
        }

        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void InvalidXMLInTgtTest()
        {

            XmlNode sessionNode;
            XmlNode mappingNode;
            navigateToMappingNode(doc, out sessionNode, out mappingNode);

            mappingNode.Attributes["tgt"].Value = InvalidVCPath;
            configFile.WriteAndLoad();
        }


        [TestMethod()]
        public void SpecialXMLCharactersInSrcTest()
        {

            XmlNode sessionNode;
            XmlNode mappingNode;
            navigateToMappingNode(doc, out sessionNode, out mappingNode);

            mappingNode.Attributes["src"].Value = mappingNode.Attributes["src"].Value + TestUtils.SpecialXMLCharacters;
            configFile.WriteAndLoad();

            VersionControlSession session = MigrationConfiguration.Current.VC.Sessions["Session0"];
            Assert.AreEqual(mappingNode.Attributes["src"].InnerText, session.Mappings[0].Source);
            Assert.AreEqual(mappingNode.Attributes["tgt"].InnerText, session.Mappings[0].Target);

        }

        [TestMethod()]
        public void SpecialXMLCharactersInTgtTest()
        {

            XmlNode sessionNode;
            XmlNode mappingNode;
            navigateToMappingNode(doc, out sessionNode, out mappingNode);

            mappingNode.Attributes["tgt"].Value = mappingNode.Attributes["tgt"].Value + TestUtils.SpecialXMLCharacters;
            configFile.WriteAndLoad();

            VersionControlSession session = MigrationConfiguration.Current.VC.Sessions["Session0"];
            Assert.AreEqual(mappingNode.Attributes["src"].InnerText, session.Mappings[0].Source);
            Assert.AreEqual(mappingNode.Attributes["tgt"].InnerText, session.Mappings[0].Target);
        }

        #endregion TestMethods
    }

}
