﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.Migration.Toolkit;
using Microsoft.TeamFoundation.Migration.Toolkit.VC;
using System.IO;
using Tests.Framework;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.VersionControl.TFSMigrationEngine&lt;T&gt; and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.VersionControl.TFSMigrationEngine&lt;T&gt; Unit Tests
    ///</summary>
    [TestClass()]
    public class SourceToTFSMigrationEngineTest : MigrationTestCaseBase
    {
        internal class TestMigrationEngine : SourceToTfsMigrationEngine
        {
            bool beforeMigrateCalled = false;
            bool afterMigrationCalled = false;
            bool sawChange = false;
            string guidToLookFor;

            internal TestMigrationEngine(string sessionId, VersionControlSession session, ChangeGroupingMananger manager)
                : base(session, manager)
            {
                this.MigrationStarting += new EventHandler<MigrationEventArgs>(TestMigrationEngine_MigrationStarting);
                this.MigrationComplete += new EventHandler<MigrationEventArgs>(TestMigrationEngine_MigrationComplete);
                guidToLookFor = sessionId;
            }

            void TestMigrationEngine_MigrationComplete(object sender, MigrationEventArgs e)
            {
                Assert.IsTrue(beforeMigrateCalled);
                Assert.IsFalse(afterMigrationCalled);

                afterMigrationCalled = true;
            }

            void TestMigrationEngine_MigrationStarting(object sender, MigrationEventArgs e)
            {
                Assert.IsFalse(beforeMigrateCalled);
                Assert.IsFalse(afterMigrationCalled);

                beforeMigrateCalled = true;
            }

            protected override void ProcessChangeGroup(ChangeGrouping group)
            {
                Assert.IsTrue(beforeMigrateCalled);
                Assert.IsFalse(afterMigrationCalled);

                if (group.SessionId == guidToLookFor)
                {
                    sawChange = true;
                }

                group.Status = ChangeStatus.Skipped;
                group.Save();
            }

            public Workspace createWSProxy()
            {
                return this.CreateWorkspace();
            }

            protected override bool IsOurTfsChanges(int changesetId)
            {
                throw new NotImplementedException();
            }
        }

        #region Additional test attributes

        //Use TestInitialize to run code before running each test
        [TestInitialize()]
        public override void Initialize()
        {
            base.Initialize();

            ConfigFile.WriteAndLoad(ConfigParameters);
            DataAccessManager.Current.CreateSchema(true);
        }

        [TestCleanup]
        public override void Cleanup()
        {
            base.Cleanup();

            string serverName = TestEnvironment.SqlServer;
            SqlUtil.DropDB(serverName, ConfigParameters.SqlInfo.Database);

        }

        #endregion


        [TestMethod]
        public void MigrationEngineDefaultEventsTest()
        {
            VersionControlSession session = new VersionControlSession();
            session.Id = Guid.NewGuid().ToString();

            ChangeGroupingMananger t2s = new SqlChangeGroupingMananger(
                null, null, session, MigrationDirection.SourceToTfs);

            ChangeGrouping grouping = t2s.Create(string.Empty);
            grouping.Status = ChangeStatus.Pending;
            grouping.ChangeTimeUtc = DateTime.Now;
            grouping.Comment = "test comment";
            grouping.SessionId = Guid.NewGuid().ToString();
            grouping.Owner = "Robert";
            grouping.Save();
            
            TestMigrationEngine engine = new TestMigrationEngine(grouping.SessionId, MigrationConfiguration.Current.VC.Sessions[ConfigParameters.VersionControl.Sessions[0].SessionId], t2s);
            engine.Run();
        }
    }
}
