﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using System.IO;
using Microsoft.TeamFoundation.Migration.Toolkit;
using Microsoft.TeamFoundation.Migration.Toolkit.VC;
using Tests.Framework;
namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.VersionControl.TFSUtil and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.VersionControl.TFSUtil Unit Tests
    ///</summary>
    [TestClass()]
    public class TFSUtilTest : MigrationTestCaseBase
    {

        [TestInitialize]
        public override void Initialize()
        {
            base.Initialize();
            ConfigFile.WriteAndLoad(ConfigParameters);
        }

        ///<summary>
        ///Scenario: Call IsValidTfsServerPath with invalid strings
        ///Expected Result: Returns false when throwOnError is false; throws MigrationException when throwOnError is true
        ///</summary>
        [TestMethod(), Priority(2), Owner("rhorvick")]
        [Description("Call IsValidTfsServerPath with invalid strings")]
        public void IsValidTfsServerPathTest()
        {
            TraceManager.AbortOnError = false;
            TraceManager.WarningsAsErrors = false;

            failBothWays(null);
            failBothWays(string.Empty);
            failBothWays(@"$/has\wrong\slashes");
            failBothWays("does not start with $/");
            failBothWays("$/" + new string('A', 270));
            failBothWays("$/segment/starts/with/$dollar");
            failBothWays("$/segment/starts/$with/dollar");

            TfsUtil.IsValidTfsServerPath("$/", true);
            TfsUtil.IsValidTfsServerPath("$/valid", true);
            TfsUtil.IsValidTfsServerPath("$/valid/also.valid", true);
            TfsUtil.IsValidTfsServerPath("$/valid/this/is/another/path.txt", true);
        }

        private void failBothWays(string path)
        {
            ensureBadPathFails(path, true);
            ensureBadPathFails(path, false);
        }

        private void ensureBadPathFails(string path, bool throwOnError)
        {
            try
            {
                bool result = TfsUtil.IsValidTfsServerPath(path, throwOnError);
                if (throwOnError)
                {
                    Assert.Fail("The bad server path should have thrown an exception");
                }

                Assert.IsFalse(result);
            }
            catch (MigrationException)
            {
                Assert.IsTrue(throwOnError);
            }
        }


        /// <summary>
        ///A test for ConcatWithoutDoubleSlashes (string, string)
        ///</summary>
        [TestMethod()]
        public void ConcatWithoutDoubleSlashesTest()
        {
            try
            {
                TfsUtil.ConcatWithoutDoubleSlashes(null, string.Empty);
                Assert.Fail("Should have thrown on null");
            }
            catch (ArgumentNullException)
            {
            }

            try
            {
                TfsUtil.ConcatWithoutDoubleSlashes(string.Empty, null);
                Assert.Fail("Should have thrown on null");
            }
            catch (ArgumentNullException)
            {
            }

            testConcat("abc", "def", "abc/def");
            testConcat("abc/", "def", "abc/def");
            testConcat("abc", "/def", "abc/def");
            testConcat("abc/", "/def", "abc/def");

            testConcat("abc", "", "abc/");
            testConcat("", "def", "/def");
            testConcat("", "", "/");
        }

        void testConcat(string p1, string p2, string expected)
        {
            string actual = TfsUtil.ConcatWithoutDoubleSlashes(p1, p2);
            Assert.AreEqual<string>(actual, expected);
        }

        /// <summary>
        ///A test for DeleteFile (string)
        ///</summary>
        [TestMethod()]
        public void DeleteFileTest()
        {
            string temp = Path.GetTempFileName();
            Assert.IsTrue(File.Exists(temp));
            TfsUtil.DeleteFile(temp);
            Assert.IsFalse(File.Exists(temp));


            temp = Path.GetTempFileName();
            Assert.IsTrue(File.Exists(temp));
            File.SetAttributes(temp, FileAttributes.ReadOnly);
            TfsUtil.DeleteFile(temp);
            Assert.IsFalse(File.Exists(temp));

            temp = Path.GetTempFileName();
            Assert.IsTrue(File.Exists(temp));
            TfsUtil.DeleteFile(temp);
            Assert.IsFalse(File.Exists(temp));
            TfsUtil.DeleteFile(temp);
            Assert.IsFalse(File.Exists(temp));
        }

        /// <summary>
        ///A test for deleteFiles (string)
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.VersionControl.dll")]
        [TestMethod()]
        public void deleteFilesTest()
        {
            string temp = Path.Combine(Path.GetTempPath(), Path.GetRandomFileName());

            Assert.IsFalse(Directory.Exists(temp));
            Directory.CreateDirectory(temp);
            Assert.IsTrue(Directory.Exists(temp));

            for (int i = 0; i < 10; i++)
            {
                string name = Path.Combine(temp, Path.GetRandomFileName());
                using (StreamWriter sw = File.CreateText(name))
                {
                    sw.WriteLine(temp);
                }

                File.SetAttributes(name, FileAttributes.ReadOnly);

                Assert.IsTrue(File.Exists(name));
            }

            Tests.Microsoft_TeamFoundation_Migration_Toolkit_TfsUtilAccessor.deleteFiles(temp);

            string[] files = Directory.GetFiles(temp);
            Assert.IsNotNull(files);
            Assert.AreEqual<int>(files.Length, 0);
        }

        /// <summary>
        ///A test for EnsurePathToFileExists (string)
        ///</summary>
        [TestMethod()]
        public void EnsurePathToFileExistsTest()
        {

            string tempDir;

            do
            {
                tempDir = Path.Combine(Path.GetTempPath(), Guid.NewGuid().ToString());
            }
            while (Directory.Exists(tempDir));

            string path = Path.Combine(tempDir, "ensurePathTest.dat");

            try
            {
                TfsUtil.EnsurePathToFileExists(path);
                Assert.IsTrue(Directory.Exists(tempDir));
            }
            finally
            {
                Directory.Delete(tempDir, true);
            }
        }

        ///<summary>
        ///Scenario: Call failWithError with a valid message
        ///Expected Result: MigrationException with the passed in message
        ///</summary>
        [TestMethod(), Priority(1), Owner("rhorvick")]
        [Description("Call failWithError with a valid message")]
        [ExpectedException(typeof(MigrationException), "some message")]
        public void FailWithErrorTest()
        {
            TfsUtil.FailWithError("some message");
        }

        ///<summary>
        ///Scenario: Call failWithError with a format string and valid message
        ///Expected Result: MigrationException with the passed in formatted message
        ///</summary>
        [TestMethod(), Priority(1), Owner("rhorvick")]
        [Description("Call failWithError with a format string and valid message")]
        [ExpectedException(typeof(MigrationException),"A error message")]
        public void FailWithErrorFormatTest()
        {
            TfsUtil.FailWithError("A {0} message","error");
        }

        ///<summary>
        ///Scenario: Call cleanWorkspace with a null workspace
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call cleanWorkspace with a null workspace")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullCleanWorkspaceTest()
        {
            TfsUtil.CleanWorkspace(null);
        }

        ///<summary>
        ///Scenario: Map a location with some files; Pend a change on an item.; Do a get on the workspace; Call clearWorkspace
        ///Expected Result: The local folders are empty after the call; There are no pending changes in the workspace.
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Map a location with some files; Pend a change on an item.; Do a get on the workspace; Call clearWorkspace")]
        public void CleanWorkspaceTest()
        {
            //MigrationConfiguration is unused in this test but leaving it in memeory locks a file in the workspace we are trying to clean
            disposeMigrationConfiguration();
            MigrationItemStrings file1 = new MigrationItemStrings("file1.txt", null, TestContext.TestName, ConfigParameters);
            MigrationItemStrings file2 = new MigrationItemStrings("file2.dat", null, TestContext.TestName, ConfigParameters);

            AddTFSFile(file1.LocalPath);
            AddTFSFile(file2.LocalPath);

            TestUtils.EditRandomFile(file1.LocalPath);
            MasterWorkspace.PendEdit(file1.LocalPath);

            MasterWorkspace.Get();

            TfsUtil.CleanWorkspace(MasterWorkspace);

            Assert.AreEqual(0, MasterWorkspace.GetPendingChanges().Length, "There should be no pending changes after CleanWorkspace");

            Assert.IsFalse(Directory.Exists(MasterWorkspace.Folders[0].LocalItem), "The working folder should not exist after cleaning");
        }

        ///<summary>
        ///Scenario: Call ConcatWithoutDoubleSlashes with null part1; Call ConcatWithoutDoubleSlashes with null part2
        ///Expected Result: ArgumentNullException 
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call ConcatWithoutDoubleSlashes with null part1; Call ConcatWithoutDoubleSlashes with null part2")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullConcatWithoutDoubleSlashesTest()
        {
            try
            {
                TfsUtil.ConcatWithoutDoubleSlashes(null, "//slashes//");
                Assert.Fail("ExpectedException AgrumentNullExcpetion");
            }
            catch (ArgumentNullException) { }
            TfsUtil.ConcatWithoutDoubleSlashes("//slashes//", null);
        }

        ///<summary>
        ///Scenario: Call DeleteFile with null path
        ///Expected Result: There are no excpetions
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call DeleteFile with null path")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullDeleteFileTest()
        {
            TfsUtil.DeleteFile(null);
        }

        ///<summary>
        ///Scenario: Call EnsurePathToFileExists with a null path; Call EnsurePathToFileExists with an empty path
        ///Expected Result: ArgumentNullException 
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call EnsurePathToFileExists with a null path; Call EnsurePathToFileExists with an empty path")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void InvalidEnsurePathToFileExistsTest()
        {
            try
            {
                TfsUtil.EnsurePathToFileExists(null);
                Assert.Fail("ExpectedException AgrumentNullExcpetion");
            }
            catch (ArgumentNullException) { }

            TfsUtil.EnsurePathToFileExists(string.Empty);
        }

        ///<summary>
        ///Scenario: Call FailWithError with a null Format string
        ///Expected Result: ArgumentNullException 
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call FailWithError with a null Format string; Call FailWithError with an empty format string")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullFormatFailWithErrorTest()
        {
            TfsUtil.FailWithError(null, "string1", "string2");
        }

        ///<summary>
        ///Scenario: Call FailWithError with a null args
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call FailWithError with a null args")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullArgsFailWithErrorTest()
        {
            TfsUtil.FailWithError("Fail with error", null);
        }

        ///<summary>
        ///Scenario: Call FailWithError with a null message; Call FailWithError with an empty message
        ///Expected Result: ArgumentNullException 
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call FailWithError with a null message; Call FailWithError with an empty message")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullMessageFailWithErrorTest()
        {
            TfsUtil.FailWithError(null);
        }

        ///<summary>
        ///Scenario: Call FindAppropriateMappingForServerPath with a null Session
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call FindAppropriateMappingForServerPath with a null Session")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullSessionFindAppropriateMappingForServerPathTest()
        {
            TfsUtil.FindAppropriateMappingForServerPath("$/path", null);
        }

        ///<summary>
        ///Scenario: Call FindAppropriateMappingForServerPath with a null ServerPath; Call FindAppropriateMappingForServerPath with an empty serverPath
        ///Expected Result: InvalidArgumentException 
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call FindAppropriateMappingForServerPath with a null ServerPath; Call FindAppropriateMappingForServerPath with an empty serverPath")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void InvalidServerPathFindAppropriateMappingForServerPathTest()
        {
            try
            {
                TfsUtil.FindAppropriateMappingForServerPath(null,CreateVcWssSession);
                Assert.Fail("ExpectedException AgrumentNullExcpetion");
            }
            catch (ArgumentNullException) { }

            TfsUtil.FindAppropriateMappingForServerPath(string.Empty, CreateVcWssSession);
        }

        ///<summary>
        ///Scenario: Call GetLocalPathForMapping with a null mapping
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call GetLocalPathForMapping with a null mapping")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullMappingGetLocalPathForMappingTest()
        {
            TfsUtil.GetLocalPathForMapping(null, CreateVcWssSession);
        }

        ///<summary>
        ///Scenario: Call GetLocalPathForMapping with a null session
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call GetLocalPathForMapping with a null session")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullSessionGetLocalPathForMappingTest()
        {
            TfsUtil.GetLocalPathForMapping(CreateVcWssSession.Mappings[0], null);
        }

        ///<summary>
        ///Scenario: Call GetLocalPathForServerItem with a null session
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call GetLocalPathForServerItem with a null session")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullSessionGetLocalPathForServerItemTest()
        {
            TfsUtil.GetLocalPathForServerItem("$/serverItem", null);
        }

        ///<summary>
        ///Scenario: Call GetLocalPathForServerItem with a null serverItem; Call GetLocalPathForServerItem with an empty serverItem
        ///Expected Result: ArgumentNullException 
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call GetLocalPathForServerItem with a null serverItem; Call GetLocalPathForServerItem with an empty serverItem")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void InvalidServerItemGetLocalPathForServerItemTest()
        {
            try
            {
                TfsUtil.GetLocalPathForServerItem(null, CreateVcWssSession);
                Assert.Fail("ExpectedException AgrumentNullExcpetion");
            }
            catch (ArgumentNullException) { }

            TfsUtil.GetLocalPathForServerItem(string.Empty, CreateVcWssSession);
        }
    }
}