﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.Migration.Toolkit;
using Microsoft.TeamFoundation.Migration.Toolkit.VC;
using Tests.Framework;
using System.IO;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.VersionControl.ChangeOptimizer and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.VersionControl.ChangeOptimizer Unit Tests
    ///</summary>
    [TestClass()]
    public class ChangeOptimizerTest : MigrationTestCaseBase
    {
        private TestAnalysisEngine engine;
        private ChangeGroupingMananger manager;
        private ChangeGrouping group;

        #region Additional test attributes
        [TestInitialize()]
        public override void Initialize()
        {
            base.Initialize();
            manager = new SqlChangeGroupingMananger(null, null, CreateVcWssSession, MigrationDirection.TfsToSource);
            engine = new TestAnalysisEngine(CreateVcWssSession, manager);
            group = CreateChangeGroup(manager);
        }

        [TestCleanup()]
        public override void Cleanup()
        {
            manager = null;
            engine = null;
            group = null;
            base.Cleanup();
        }

        #endregion

        #region AddTest
        /// <summary>
        ///A test for PendAdd (TFSChangeItem)
        ///</summary>
        [TestMethod()]
        public void AddTest()
        {
            simpleAddTest();
            addTargetIsRenameSource();
            introducedChangeTest();
            notAcceptingNewChanges();
            sameTargetConflict();
            sameSourceConflict();
            knownSourceMatchesCurrentTarget();
            longCycleOfRenames();
            longCycleOfMerges();
            deleteAndAddTest();
            undeleteAndEditTest();
        }

        void simpleAddTest()
        {
            // simple add case
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));
            target.Add(new BatchedItem("$/TestProject/File2.txt", ChangeAction.Add));
            target.Add(new BatchedItem("$/TestProject/File3.txt", ChangeAction.Add));

            ReadOnlyCollection<BatchedItem> changes = target.Resolve();
            Assert.IsNotNull(changes);
            Assert.AreEqual<int>(changes.Count, 3);
        }

        void introducedChangeTest()
        {
            // conflicting renames should introduce an intermediate action - so two in, 3 out.
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", "$/TestProject/File2.txt", ChangeAction.Rename));
            target.Add(new BatchedItem("$/TestProject/File2.txt", "$/TestProject/File1.txt", ChangeAction.Rename));

            ReadOnlyCollection<BatchedItem> changes = target.Resolve();
            Assert.IsNotNull(changes);
            Assert.AreEqual<int>(changes.Count, 3);
        }

        void notAcceptingNewChanges()
        {
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));
            target.Resolve();

            try
            {
                target.Add(new BatchedItem("$/TestProject/File2.txt", ChangeAction.Add));
                Assert.Fail("PendAdd on locked change optimizer should have thrown an InvalidOperationException");
            }
            catch (InvalidOperationException)
            {
            }
        }

        void sameTargetConflict()
        {
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));

            try
            {
                target.Resolve();
                Assert.Fail("Resolve on two items with the same target is a conflict");
            }
            catch (UnresolvableConflictException)
            {
            }
        }

        void deleteAndAddTest()
        {
            ChangeOptimizer target = new ChangeOptimizer();

            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Delete));

            ReadOnlyCollection<BatchedItem> items = target.Resolve();
            Assert.AreEqual<int>(items.Count, 2);
            Assert.AreEqual<ChangeAction>(items[0].Action, ChangeAction.Add);
            Assert.AreEqual<ChangeAction>(items[1].Action, ChangeAction.Delete);
        }

        void undeleteAndEditTest()
        {
            ChangeOptimizer target = new ChangeOptimizer();

            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Edit));
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Undelete));

            ReadOnlyCollection<BatchedItem> items = target.Resolve();
            Assert.AreEqual<int>(items.Count, 2);
            Assert.AreEqual<ChangeAction>(items[0].Action, ChangeAction.Undelete);
            Assert.AreEqual<ChangeAction>(items[1].Action, ChangeAction.Edit);
        }

        void addTargetIsRenameSource()
        {
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));
            target.Add(new BatchedItem("$/TestProject/File1.txt", "$/TestProject/File2.txt", ChangeAction.Rename, 0));

            ReadOnlyCollection<BatchedItem> changes = target.Resolve();

            // there should not be an introduced change
            Assert.AreEqual<int>(changes.Count, 2);

            // this should re-order the changes such that the Rename comes before the Add.
            Assert.AreEqual<ChangeAction>(changes[0].Action, ChangeAction.Rename);
            Assert.AreEqual<ChangeAction>(changes[1].Action, ChangeAction.Add);

            // the priorities should be 0 and 1
            Assert.AreEqual<int>(changes[0].Priority, 0);
            Assert.AreEqual<int>(changes[1].Priority, 1);
        }


        void sameSourceConflict()
        {
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));

            try
            {
                target.Resolve();
                Assert.Fail("Resolve on two items with the same source is a conflict");
            }
            catch (UnresolvableConflictException)
            {
            }
        }

        void knownSourceMatchesCurrentTarget()
        {
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", ChangeAction.Add));
            target.Add(new BatchedItem("$/TestProject/File1.txt", "$/TestProject/File2.txt", ChangeAction.Rename, 0));

            target.Resolve();
        }

        void longCycleOfRenames()
        {
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", "$/TestProject/File2.txt", ChangeAction.Rename, 0));
            target.Add(new BatchedItem("$/TestProject/File2.txt", "$/TestProject/File3.txt", ChangeAction.Rename, 0));
            target.Add(new BatchedItem("$/TestProject/File3.txt", "$/TestProject/File4.txt", ChangeAction.Rename, 0));
            target.Add(new BatchedItem("$/TestProject/File4.txt", "$/TestProject/File1.txt", ChangeAction.Rename, 0));

            ReadOnlyCollection<BatchedItem> changes = target.Resolve();
            Assert.AreEqual<int>(changes.Count, 8);
        }

        void longCycleOfMerges()
        {
            ChangeOptimizer target = new ChangeOptimizer();
            target.Add(new BatchedItem("$/TestProject/File1.txt", "$/TestProject/File2.txt", ChangeAction.Merge, 0));
            target.Add(new BatchedItem("$/TestProject/File2.txt", "$/TestProject/File3.txt", ChangeAction.Merge, 0));
            target.Add(new BatchedItem("$/TestProject/File3.txt", "$/TestProject/File4.txt", ChangeAction.Merge, 0));
            target.Add(new BatchedItem("$/TestProject/File4.txt", "$/TestProject/File1.txt", ChangeAction.Merge, 0));

            ReadOnlyCollection<BatchedItem> changes = target.Resolve();
            Assert.AreEqual<int>(changes.Count, 4);
        }
        #endregion

        #region Unit Tests
        ///<summary>
        ///Scenario: Call Add passing in null
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call Add passing in null")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AddNullTest()
        {
            ChangeOptimizer opt = new ChangeOptimizer();
            opt.Add(null);
        }

        ///<summary>
        ///Scenario: Call resolve without having added any GenericChangeItems
        ///Expected Result: Returns empty list
        ///</summary>
        [TestMethod(), Priority(2), Owner("curtisp")]
        [Description("Call resolve without having added any GenericChangeItems")]
        public void EmptyResolveTest()
        {
            ChangeOptimizer optimizer = new ChangeOptimizer();
            Assert.AreEqual(0, optimizer.Resolve().Count, "Resolve should have returned an empty list");
        }

        ///<summary>
        ///Scenario: Call clear on a ChangeOptimizer that is not accepting changes and contains resolved and unresolved changes
        ///Expected Result: Resolved and Unresolved changes are cleared. ChangeOptimizer accepts changes again
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Call clear on a ChangeOptimizer that is not accepting changes and contains resolved and unresolved changes")]
        public void ClearTest()
        {
            ChangeOptimizer optimizer = new ChangeOptimizer();
            optimizer.Add(new BatchedItem("Change1", ChangeAction.Unknown));
            optimizer.Add(new BatchedItem("Change2", ChangeAction.Unknown));

            optimizer.Resolve();
            Microsoft_TeamFoundation_Migration_Toolkit_ChangeOptimizerAccessor accessor = new Microsoft_TeamFoundation_Migration_Toolkit_ChangeOptimizerAccessor(optimizer);
            accessor.acceptingNewChanges = true;
            optimizer.Add(new BatchedItem("Change3", ChangeAction.Unknown));

            accessor.acceptingNewChanges = false;

            optimizer.Clear();

            Assert.IsTrue(accessor.acceptingNewChanges, "Clear should have opened the optimizer for changes");
            Assert.AreEqual(0, optimizer.Resolve().Count, "There should be no resolved changes after clear");
        }

        #endregion
    }
}