﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using Microsoft.TeamFoundation.Migration.Toolkit;
using System.Xml;
using System.Xml.Schema;
using Tests.Framework;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.Toolkit.MigrationProvider and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.Toolkit.MigrationProvider Unit Tests
    ///</summary>
    [TestClass()]
    public class MigrationProviderTest : ConfigManagerTestCaseBase
    {


        #region Test Cases

        [TestMethod()]
        public void MainLineMigrationProviderTest()
        {
            XmlNode node = doc.SelectSingleNode("/Migration/Providers");

            configFile.WriteAndLoad();

            foreach (XmlNode providerNode in node.ChildNodes)
            {
                MigrationProvider provider;
                String id = providerNode.Attributes["id"].InnerText;
                MigrationConfiguration.Providers.TryGetValue(id, out provider);

                Assert.AreEqual(providerNode.FirstChild.InnerText, provider.AssemblyQualifiedName);
            }
        }

        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void EmptyAssemblyQualifiedNameTest()
        {
            XmlNode node = doc.SelectSingleNode("/Migration/Providers/Provider/AssemblyQualifiedName");

            node.InnerText = string.Empty;
            configFile.WriteAndLoad();
        }

        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void MissingAssemblyQualifiedNameTest()
        {
            XmlNode node = doc.SelectSingleNode("/Migration/Providers/Provider");
            node = node.RemoveChild(doc.SelectSingleNode("/Migration/Providers/Provider/AssemblyQualifiedName"));

            configFile.WriteAndLoad();
        }

        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void NullMigrationProviderIdTest()
        {
            XmlNode node = doc.SelectSingleNode("/Migration/Providers/Provider");
            node.Attributes.Remove(node.Attributes["id"]);

            configFile.WriteAndLoad();
        }

        [TestMethod()]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void EmptyMigrationProviderIdTest()
        {
            XmlNode node = doc.SelectSingleNode("/Migration/Providers/Provider");
            node.Attributes["id"].Value = String.Empty;

            configFile.WriteAndLoad();
        }

        [TestMethod()]
        [ExpectedException(typeof(InitializationException))]
        public void UnknownProviderTypeTest()
        {
            XmlNode node = doc.SelectSingleNode("/Migration/Providers/Provider/AssemblyQualifiedName");

            node.InnerText = "BogusProvider";
            configFile.WriteAndLoad();
        }

    }
        #endregion Test cases

}