﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using System.IO;
using Microsoft.TeamFoundation.Migration.Toolkit;
using System.Data.SqlClient;
using System.Data;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Schema;
using Tests.Framework;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.Toolkit.Reporting.SqlReportAdaptor and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.Toolkit.Reporting.SqlReportAdaptor Unit Tests
    ///</summary>
    [TestClass()]
    public class SQLReportAdaptorTest : ReportAdaptorTestCaseBase
    {
        #region Additional test attributes
        private int getCount(string connStr, string db, string table)
        {
            using (SqlConnection connection = new SqlConnection(connStr))
            {
                connection.Open();
                connection.ChangeDatabase(db);
                using (SqlCommand command = connection.CreateCommand())
                {

                    command.CommandText = "SELECT COUNT(*) FROM dbo." + table;
                    command.CommandType = CommandType.Text;

                    return (int)command.ExecuteScalar();
                }
            }
        }

        private void QueryResultAsExpected(String database, String variableUnderTest, String expectedResult)
        {
            String connectionString = string.Format("server={0};Integrated Security=SSPI", serverName, database);

            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                connection.Open();
                connection.ChangeDatabase(database);
                using (SqlCommand command = connection.CreateCommand())
                {

                    command.CommandText = String.Format("SELECT {0} FROM dbo.Events WHERE {1} = '{2}'", variableUnderTest, variableUnderTest, expectedResult);
                    command.CommandType = CommandType.Text;

                    Assert.AreEqual(expectedResult, command.ExecuteScalar());
                }
            }
        }


        private void prepareSQLAdaptorXMLFile()
        {
            XmlNode parent = doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink");
            parent.RemoveAll();

            XmlElement element = parent as XmlElement;
            element.SetAttribute("provider", "SQLReport");
            XmlElement sqlNode = doc.CreateElement("SQLReport");

            XmlElement connectionStringNode = doc.CreateElement("ConnectionString");
            connectionStringNode.InnerText = "server=" + serverName + ";Integrated Security=SSPI";
            sqlNode.AppendChild(connectionStringNode);

            XmlElement databaseNode = doc.CreateElement("Database");
            databaseNode.InnerText = database;
            sqlNode.AppendChild(databaseNode);
            parent.AppendChild(sqlNode);
        }

        private void runSQLTest(MigrationSessionEventArgs args, String variableUnderTest, String expectedResult)
        {
            using (VersionControlSession session = new VersionControlSession())
            using (SqlReportAdaptor sqlReport = new SqlReportAdaptor())
            {
                SqlUtil.DropDB(serverName, database);
                String connectionString = string.Format("server={0};Integrated Security=SSPI", serverName, database);

                sqlReport.ConnectionString = connectionString;
                sqlReport.Database = database;
                sqlReport.RegisterSession(session);

                session.OnSessionStart(args);

                using (SqlConnection connection = new SqlConnection(connectionString))
                {
                    connection.Open();
                    connection.ChangeDatabase(database);
                    using (SqlCommand command = connection.CreateCommand())
                    {

                        command.CommandText = String.Format("SELECT {0} FROM dbo.Events", variableUnderTest);
                        command.CommandType = CommandType.Text;

                        object res = command.ExecuteScalar();
                        Assert.IsTrue(
                            res is DBNull && expectedResult == null || res.Equals(expectedResult),
                            "Values do not match!");
                    }
                }
            }
        }


        //Use TestInitialize to run code before running each test
        [TestInitialize()]
        public override void Initialize()
        {
            base.Initialize();
            database = TestContext.TestName;
            serverName = TestEnvironment.SqlServer;

        }

        //Use TestCleanup to run code after each test has run
        [TestCleanup()]
        public override void Cleanup()
        {
            SqlUtil.DropDB(serverName, database);
            base.Cleanup();
        }

        private string serverName;
        private string database;

        #endregion

        #region Tests on SQLAdaptor
        /// <summary>
        ///A test for validateLoggingDB ()
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.Toolkit.dll")]
        [TestMethod, Description("Register Session, Events are fired to SQLReportAdaptor, Database does not exist, Database exists and contains the appropriate resources")]
        public void SQLReportAdaptor_End2End()
        {
            // look for a custom test sql server or default to localhost
            SqlUtil.DropDB(serverName, database);

            prepareSQLAdaptorXMLFile();
            configFile.WriteXMLFile(null);

            using (FileStream fs = new FileStream(configFileName, FileMode.Open))
            {
                using (MigrationConfiguration config = MigrationConfiguration.Load(fs))
                {
                    SimulateVCSession(config, "Session0");
                }
                fs.Seek(0, SeekOrigin.Begin);

                // second time the DB does (and should be reused)
                using (MigrationConfiguration config = MigrationConfiguration.Load(fs))
                {
                    SimulateVCSession(config, "Session0");
                }
                // verify results here
                QueryResultAsExpected(database, "Description", Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionStarting);
            }
        }

        /// <summary>
        ///A test for validate server name, no connection string
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.Toolkit.dll")]
        [TestMethod, Description("Server and database specified without connection")]
        public void SQLReportAdaptorValidDBServerOnly()
        {
            SqlUtil.DropDB(serverName, database);

            prepareSQLAdaptorXMLFile();
            XmlNode parent = doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink");
            parent.RemoveAll();

            XmlElement element = parent as XmlElement;
            element.SetAttribute("provider", "SQLReport");
            XmlElement sqlNode = doc.CreateElement("SQLReport");

            XmlElement serverNode = doc.CreateElement("Server");
            serverNode.InnerText = serverName;
            sqlNode.AppendChild(serverNode);

            XmlElement databaseNode = doc.CreateElement("Database");
            databaseNode.InnerText = database;
            sqlNode.AppendChild(databaseNode);

            parent.AppendChild(sqlNode);
            configFile.WriteXMLFile(null);

            using (FileStream fs = new FileStream(configFileName, FileMode.Open))
            {
                using (MigrationConfiguration config = MigrationConfiguration.Load(fs))
                {
                    SimulateVCSession(config, "Session0");
                }
            }
            QueryResultAsExpected(database, "Description", Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionStarting);
        }

        /// <summary>
        ///Connection String specified; server and database specified
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.Toolkit.dll")]
        [TestMethod, Description("Connection String specified; server and database specified")]
        public void SQLReportAdaptorInValidDBServerValidConnectionString()
        {
            SqlUtil.DropDB(serverName, database);

            prepareSQLAdaptorXMLFile();
            XmlNode parent = doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink");
            parent.RemoveAll();

            XmlElement element = parent as XmlElement;
            element.SetAttribute("provider", "SQLReport");
            XmlElement sqlNode = doc.CreateElement("SQLReport");

            XmlElement connectionStringNode = doc.CreateElement("ConnectionString");
            connectionStringNode.InnerText = "server=" + serverName + ";Initial Catalog=" + database + ";Integrated Security=SSPI";
            sqlNode.AppendChild(connectionStringNode);

            XmlElement serverNode = doc.CreateElement("Server");
            serverNode.InnerText = serverName;
            sqlNode.AppendChild(serverNode);

            XmlElement databaseNode = doc.CreateElement("Database");
            databaseNode.InnerText = "invalidDBName";
            sqlNode.AppendChild(databaseNode);

            parent.AppendChild(sqlNode);
            configFile.WriteXMLFile(null);

            using (FileStream fs = new FileStream(configFileName, FileMode.Open))
            {
                using (MigrationConfiguration config = MigrationConfiguration.Load(fs))
                {
                    TextReportAdaptorTest.SimulateVCSession(config, "Session0");
                }
            }
            QueryResultAsExpected(database, "Description", Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionStarting);
        }


        public class SQLResult
        {
            public  int id;
            public string description;
            public DateTime dateTime;
            public string sessionId;
            public string exception;
            public string sessionType;
            public string systemType;

            public SQLResult(int id, string description, DateTime dateTime)
            {
                this.id = id;
                this.description = description;
                this.dateTime = dateTime;
            }

            public SQLResult(int id, string description, DateTime dateTime, string sessionId, string sessionType)
                : this(id, description, dateTime)
            {
                this.sessionId = sessionId;
                this.sessionType = sessionType;
            }

            public SQLResult(int id, string description, DateTime dateTime, string sessionId, string sessionType, string systemType)
                : this(id, description, dateTime)
            {
                this.sessionId = sessionId;
                this.sessionType = sessionType;
                this.systemType = systemType;
            }


        }

        /// <summary>
        ///Register same adaptor on 2 sessions
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.Toolkit.dll")]
        [TestMethod, Description("Register same adaptor on 2 sessions")]
        public void RegisterAdaptorOnTwoSessions()
        {
            VersionControlSession session1 = new VersionControlSession();
            session1.Id = "Session0";

            VersionControlSession session2 = new VersionControlSession();
            session2.Id = "Session1";

            using (SqlReportAdaptor sqlAdaptor = new SqlReportAdaptor())
            {
                Assert.IsNotNull(serverName);
                SqlUtil.DropDB(serverName, database);

                sqlAdaptor.ConnectionString = string.Format("server={0};Integrated Security=SSPI", serverName);
                sqlAdaptor.Database = database;
                sqlAdaptor.RegisterSession(session1);
                sqlAdaptor.RegisterSession(session2);

                List<SQLResult> events = new List<SQLResult>();


                session1.Start();
                events.Add(new SQLResult(1, Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionStarting, DateTime.Now.ToUniversalTime(), session1.Id, session1.GetType().AssemblyQualifiedName));

                session2.Start();
                events.Add(new SQLResult(2, Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionStarting, DateTime.Now.ToUniversalTime(), session2.Id, session2.GetType().AssemblyQualifiedName));

                session1.OnAnalyzingChangeStarting(new VersionControlEventArgs("Change 1", SystemType.Other));
                events.Add(new SQLResult(3, "Change 1", DateTime.Now.ToUniversalTime(), session1.Id, session1.GetType().AssemblyQualifiedName, SystemType.Other.ToString()));

                session2.OnAnalyzingChangeStarting(new VersionControlEventArgs("Change 2", SystemType.Tfs));
                events.Add(new SQLResult(4, "Change 2", DateTime.Now.ToUniversalTime(), session2.Id, session2.GetType().AssemblyQualifiedName, SystemType.Tfs.ToString()));

                session1.Stop();
                events.Add(new SQLResult(5, Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionComplete, DateTime.Now.ToUniversalTime(), session1.Id, session1.GetType().AssemblyQualifiedName));

                session2.Stop();
                events.Add(new SQLResult(6, Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionComplete, DateTime.Now.ToUniversalTime(), session2.Id, session2.GetType().AssemblyQualifiedName));

                using (SqlConnection connection = new SqlConnection(sqlAdaptor.ConnectionString))
                {
                    connection.Open();
                    connection.ChangeDatabase(sqlAdaptor.Database);
                    foreach (SQLResult savedEvent in events)
                    {

                        using (SqlCommand command = connection.CreateCommand())
                        {
                            command.CommandText = string.Format("SELECT Description FROM dbo.Events WHERE id='{0}';",savedEvent.id);
                            command.CommandType = CommandType.Text;
                            Assert.AreEqual(savedEvent.description, command.ExecuteScalar());

                            command.CommandText = string.Format("SELECT UTCWhen FROM dbo.Events WHERE id='{0}';", savedEvent.id);
                            command.CommandType = CommandType.Text;
                            Assert.AreEqual(savedEvent.dateTime.ToString(), ((DateTime)command.ExecuteScalar()).ToString());


                            command.CommandText = string.Format("SELECT UTCWhen FROM dbo.Events WHERE id='{0}';", savedEvent.id);
                            command.CommandType = CommandType.Text;
                            Assert.AreEqual(savedEvent.dateTime.ToString(), ((DateTime)command.ExecuteScalar()).ToString());

                            if (savedEvent.systemType == null)
                            {
                                command.CommandText = string.Format("SELECT SessionId FROM dbo.SessionEvents WHERE id='{0}';", savedEvent.id);
                                command.CommandType = CommandType.Text;
                                Assert.AreEqual(savedEvent.sessionId, command.ExecuteScalar());

                                command.CommandText = string.Format("SELECT SessionType FROM dbo.SessionEvents WHERE id='{0}';", savedEvent.id);
                                command.CommandType = CommandType.Text;
                                Assert.AreEqual(savedEvent.sessionType, command.ExecuteScalar());
                            }
                            else
                            {
                                command.CommandText = string.Format("SELECT SessionId FROM dbo.VersionControlEvents WHERE id='{0}';", savedEvent.id);
                                command.CommandType = CommandType.Text;
                                Assert.AreEqual(savedEvent.sessionId, command.ExecuteScalar());

                                command.CommandText = string.Format("SELECT PrimarySystem FROM dbo.VersionControlEvents WHERE id='{0}';", savedEvent.id);
                                command.CommandType = CommandType.Text;
                                Assert.AreEqual(savedEvent.systemType, command.ExecuteScalar());
                            }

                        }
                    }
                }
            }
        }

        [TestMethod]
        [ExpectedException(typeof(SqlException))]
        public void SQLExceptionTest()
        {
            VersionControlSession session = new VersionControlSession();

            using (SqlReportAdaptor sqlAdaptor = new SqlReportAdaptor())
            {
                string username = "BogusUserName";
                string password = "BadPassword";

                sqlAdaptor.ConnectionString = string.Format("server={0};Database={1};User ID={2};Password={3};Trusted_Connection=False", serverName, database, username, password);

                sqlAdaptor.Database = database;
                sqlAdaptor.RegisterSession(session);
            }
        }

        /// <summary>
        ///Same session has 2 adaptors registered
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.Toolkit.dll")]
        [TestMethod, Description("Same session has 2 adaptors registered")]
        public void TestTwoAdaptorsInOneSession()
        {
            configFile.WriteAndLoad();

            VersionControlSession session = MigrationConfiguration.Current.VC.Sessions["Session0"];
            Assert.IsNotNull(session);

            string db1 = "MigrationToolkitTest1";
            string db2 = "MigrationToolkitTest2";
            try
            {
                using (SqlReportAdaptor sqlAdaptor1 = new SqlReportAdaptor())
                using (SqlReportAdaptor sqlAdaptor2 = new SqlReportAdaptor())
                {
                    Assert.IsNotNull(serverName);

                    sqlAdaptor1.ConnectionString = string.Format("server={0};Integrated Security=SSPI", serverName);
                    sqlAdaptor1.Database = db1;
                    sqlAdaptor1.RegisterSession(session);

                    sqlAdaptor2.ConnectionString = string.Format("server={0};Integrated Security=SSPI", serverName);
                    sqlAdaptor2.Database = db2;
                    sqlAdaptor2.RegisterSession(session);
                    Assert.IsFalse(session.IsRunning);
                    Assert.IsFalse(session.IsComplete);

                    MigrationConfiguration.Current.VC.Start();

                    string desc = "Analyzing change";
                    session.OnAnalyzingChangeStarting(new VersionControlEventArgs(desc, SystemType.Other));

                    MigrationConfiguration.Current.VC.Stop();

                    Assert.IsTrue(session.IsComplete);
                    Assert.IsFalse(session.IsRunning);

                    QueryResultAsExpected(sqlAdaptor1.Database, "Description", desc);
                    QueryResultAsExpected(sqlAdaptor2.Database, "Description", desc);
                }
            }
            finally
            {
                SqlUtil.DropDB(serverName, db1);
                SqlUtil.DropDB(serverName, db2);
            }
        }

        /// <summary>
        ///SqlReportAdaptor is disposed without any events fired
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.Toolkit.dll")]
        [TestMethod, Description("SqlReportAdaptor is disposed without any events fired")]
        public void SqlReportAdaptorDisposeWithoutEventFired()
        {
            VersionControlSession session = new VersionControlSession();

            using (SqlReportAdaptor sqlAdaptor = new SqlReportAdaptor())
            {
                SqlUtil.DropDB(serverName, database);
                String connectionString = string.Format("server={0};Integrated Security=SSPI", serverName);

                sqlAdaptor.ConnectionString = connectionString;
                sqlAdaptor.Database = database;
                sqlAdaptor.RegisterSession(session);

                //verify there is no record in the event log table
                Assert.AreEqual(0, getCount(connectionString, database, "Events"));
            }
        }


        #endregion Tests on SQLAdaptor

        #region MigrationEventArgs tests on SQLAdaptor

        [TestMethod]
        public void SpecialXmlCharactersInDescriptionTest()
        {
            MigrationSessionEventArgs args = new MigrationSessionEventArgs(TestUtils.SpecialXMLCharacters);

            String variableUnderTest = "Description";
            String expectedResult = TestUtils.SpecialXMLCharacters;

            String connectionString = string.Format("server={0};database={1};Integrated Security=SSPI", serverName, database);

            runSQLTest(args, variableUnderTest, expectedResult);
        }

        [TestMethod]
        public void InvalidXmlCharactersInDescriptionTest()
        {
            MigrationSessionEventArgs args = new MigrationSessionEventArgs(TestUtils.InvalidXMLCharacters);

            String variableUnderTest = "Description";
            String expectedResult = TestUtils.InvalidXMLCharacters;

            runSQLTest(args, variableUnderTest, expectedResult);

        }

        [TestMethod]
        public void NullDescriptionTest()
        {
            MigrationSessionEventArgs args = new MigrationSessionEventArgs(null);

            String variableUnderTest = "Description";
            String expectedResult = null;
            runSQLTest(args, variableUnderTest, expectedResult);
        }

        [TestMethod]
        public void NullExceptionTest()
        {
            MigrationSessionEventArgs args = new MigrationSessionEventArgs(initializer.validDescription(), initializer.validLong(), initializer.validLong(), null);
            String variableUnderTest = "Exception";


            SqlUtil.DropDB(serverName, database);
            String connectionString = string.Format("server={0};Integrated Security=SSPI", serverName, database);

            using (SqlReportAdaptor sqlReport = new SqlReportAdaptor())
            using (VersionControlSession session = new VersionControlSession())
            {

                sqlReport.ConnectionString = connectionString;
                sqlReport.Database = database;
                sqlReport.RegisterSession(session);

                session.OnSessionStart(args);

                using (SqlConnection connection = new SqlConnection(connectionString))
                {
                    connection.Open();
                    connection.ChangeDatabase(database);
                    using (SqlCommand command = connection.CreateCommand())
                    {

                        command.CommandText = String.Format("SELECT {0} FROM dbo.Events", variableUnderTest);
                        command.CommandType = CommandType.Text;

                        Assert.IsInstanceOfType(command.ExecuteScalar(), typeof(DBNull));
                    }
                }
            }
        }

        #endregion MigrationEventArgs tests on SQLAdaptor
    }
}
