﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using System.Globalization;
using Microsoft.TeamFoundation.Migration.Toolkit;
using System.IO;
using System.Xml;
using System.Text.RegularExpressions;
using Tests.Framework;
namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.Toolkit.TextReportAdaptor and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.Toolkit.TextReportAdaptor Unit Tests
    ///</summary>
    [TestClass()]
    public class TextReportAdaptorTest : ReportAdaptorTestCaseBase
    {

        #region Additional test attributes


        private static String readOutputLine(String filePath)
        {
            FileStream fs = new FileStream(filePath, FileMode.Open);
            StreamReader sr = new StreamReader(fs);
            String firedEvent = sr.ReadLine();
            sr.Dispose();
            return firedEvent;
        }
        #endregion

        #region TextAdaptorTests
        /// <summary>
        ///A test for SessionEvent (object, MigrationEventArgs)
        ///</summary>
        [TestMethod()]
        public void TextReportAdaptorQuickTest()
        {
            using (Stream configFile = this.GetType().Assembly.GetManifestResourceStream("Tests.ConfigurationManager.Test1.xml"))
            {
                using (MigrationConfiguration config = MigrationConfiguration.Load(configFile))
                {
                    SimulateVCSession(config, "Session1");
                }
            }
        }

        [TestMethod]
        public void TrueAppendTest()
        {

            String startText = Guid.NewGuid().ToString();

            String textReportFilePath = Path.GetTempFileName();

            doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink/TextReport/File").InnerText = textReportFilePath;
            doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink/TextReport/Append").InnerText = "true";

            using (StreamWriter sw = new StreamWriter(textReportFilePath))
            {
                sw.WriteLine(startText);
            }

            try
            {
                configFile.WriteAndLoad();

                //Fire some events to the text report
                MigrationConfiguration.Current.VC.Start();

                disposeMigrationConfiguration();

                using (StreamReader sr = new StreamReader(textReportFilePath))
                {
                    // Check that the text wasnt overwritten
                    Assert.AreEqual(startText, sr.ReadLine());
                }
            }
            finally
            {
                File.Delete(textReportFilePath);
            }
        }

        [TestMethod]
        public void MainLineTextReportAdaptorTest()
        {

            String textReportFilePath = Environment.GetEnvironmentVariable("temp") + "\\MainLineTextReportAdaptorTest.txt";
            doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink/TextReport/File").InnerText = textReportFilePath;

            configFile.WriteAndLoad();

            DateTime[] savedTimes = new DateTime[4];
            String[] eventMessages = new string[4];
            VersionControlSession session = MigrationConfiguration.Current.VC.Sessions["Session0"];
            String Sessionid = session.Id;

            savedTimes[0] = DateTime.Now;
            session.Start();
            eventMessages[0] = Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionStarting;

            savedTimes[1] = DateTime.Now;
            session.Abort();
            eventMessages[1] = Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionAborted;

            savedTimes[2] = DateTime.Now;
            session.Start();
            eventMessages[2] = Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionStarting;

            savedTimes[3] = DateTime.Now;
            session.Stop();
            eventMessages[3] = Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.SessionComplete;


            disposeMigrationConfiguration();
            FileStream fs = new FileStream(textReportFilePath, FileMode.Open);
            StreamReader sr = new StreamReader(fs);
            for (int i = 0; i < savedTimes.Length; i++)
            {
                Regex exp = new Regex(String.Format(@"\[\d+, \d+, {0}, VC Session: {1}] {2}", savedTimes[i].ToString(CultureInfo.CurrentCulture), Sessionid, eventMessages[i]));
                String firedEvent = sr.ReadLine();
                // Check that the text wasnt overwritten
                Assert.IsTrue(exp.IsMatch(firedEvent), String.Format("{0} did not match {1}", firedEvent, exp.ToString()));

            }
            Assert.IsTrue(sr.EndOfStream, "Extra data at the end of the file");
            sr.Dispose();
            File.Delete(textReportFilePath);
        }

        [TestMethod]
        public void FalseAppendTest()
        {

            String startText = "This String will be erased";

            String textReportFilePath = Environment.GetEnvironmentVariable("temp") + "\\FalseAppendTest.txt";
            doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink/TextReport/File").InnerText = textReportFilePath;

            FileStream fs = new FileStream(textReportFilePath, FileMode.OpenOrCreate);

            using (StreamWriter sw = new StreamWriter(fs))
            {
                // Add some text to the file.
                sw.WriteLine(startText);
            }

            configFile.WriteAndLoad();

            //Fire some events to the text report
            MigrationConfiguration.Current.VC.Start();

            disposeMigrationConfiguration();

            fs = new FileStream(textReportFilePath, FileMode.Open);

            using (StreamReader sr = new StreamReader(fs))
            {
                // Check that the text wasnt overwritten
                Assert.AreNotEqual(startText, sr.ReadLine());
            }
            File.Delete(textReportFilePath);
        }

        [TestMethod]
        public void ReuseAdaptorTest()
        {
            using (TextReportAdaptor sink = new TextReportAdaptor())
            {
                sink.File = Environment.GetEnvironmentVariable("temp") + "test.txt";

                using (VersionControlSession session = new VersionControlSession())
                using (VersionControlSession anotherSession = new VersionControlSession())
                {

                    session.RegisterEventSink(sink);
                    try
                    {
                        anotherSession.RegisterEventSink(sink);
                        Assert.Fail("Reusing a TextReport should have thrown an exception");
                    }
                    catch (MigrationException e)
                    {
                        Assert.AreEqual(Microsoft_TeamFoundation_Migration_Toolkit_MigrationToolkitResourcesAccessor.OneSessionPerTextReport, e.Message);
                    }
                }
            }
        }

        [TestMethod]
        public void DualAdaptorTest()
        {
            String root = Environment.GetEnvironmentVariable("temp") + "\\DualAdaptorTest\\";

            string firstFile = root + "firstReport.txt";
            string secondFile = root + "secondReport.txt";

            try
            {
                configFile.WriteAndLoad();

                using (TextReportAdaptor firstTextReport = new TextReportAdaptor())
                using (TextReportAdaptor secondTextReport = new TextReportAdaptor())
                {
                    firstTextReport.File = firstFile;
                    secondTextReport.File = secondFile;

                    VersionControlSession session = MigrationConfiguration.Current.VC.Sessions["Session0"];
                    session.RegisterEventSink(firstTextReport);
                    session.RegisterEventSink(secondTextReport);

                    SimulateVCSession(MigrationConfiguration.Current, "Session0");
                }
            }
            finally
            {
                MigrationConfiguration.Current.Dispose();
            }

            using (StreamReader firstReportReader = new StreamReader(firstFile))
            using (StreamReader secondReportReader = new StreamReader(secondFile))
            {
                //Check that the contents are the same
                while (!firstReportReader.EndOfStream)
                {
                    Assert.AreEqual(firstReportReader.ReadLine(), secondReportReader.ReadLine(), "These Lines should match");
                }

                Assert.IsTrue(secondReportReader.EndOfStream, "The second report was longer than the first");
            }

            Directory.Delete(root, true);
        }


        [TestMethod]
        public void NoEventsTest()
        {

            String textReportFilePath = Environment.GetEnvironmentVariable("temp") + "\\NoEventsTest.txt";
            doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink/TextReport/File").InnerText = textReportFilePath;

            configFile.WriteAndLoad();
            //stop without firing events
            disposeMigrationConfiguration();

            FileStream fs = new FileStream(textReportFilePath, FileMode.Open);
            StreamReader sr = new StreamReader(fs);

            Assert.IsTrue(sr.EndOfStream, "This file should be empty");
            sr.Dispose();
            File.Delete(textReportFilePath);

        }

        //Since the toolkit should be making no effort to handle IOExceptions any iOException is logicaly equivilent to the next. 
        //So this test will cover cases like out of disk space and incorrect permissions as well as being unable to create the file.
        [TestMethod]
        public void IOExceptionTest()
        {


            //Save the file path
            String filepath = doc.SelectSingleNode("/Migration/VC/Sessions/Session/EventSinks/EventSink/TextReport/File").InnerText;

            //Make sure the fileLock and find the path
            Directory.CreateDirectory(TestUtils.TextReportRoot);
            //Lock the File
            StreamWriter fileLock = new StreamWriter(new FileStream(filepath, FileMode.CreateNew, FileAccess.Write, FileShare.None));

            try
            {
                configFile.WriteAndLoad();
                Assert.Fail("The file is locked, load should have thrown an IOException");
            }
            catch (InitializationException e)
            {
                IOException io = new IOException();
                Assert.IsInstanceOfType(e.InnerException, io.GetType());
            }
            fileLock.Dispose();
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullSessionTest()
        {
            using (TextReportAdaptor textReport = new TextReportAdaptor())
            {
                textReport.RegisterSession(null);
            }
        }


        ///<summary>
        ///Scenario: Create a text report on a realative path
        ///Expected Result: text report is created
        ///</summary>
        [TestMethod(), Priority(2), Owner("curtisp")]
        [Description("Create a text report on a realative path")]
        public void EmptyPathTestTest()
        {
            string fileName = "TextReport.txt";
            try
            {
                using (TextReportAdaptor textReport = new TextReportAdaptor())
                {
                    textReport.File = fileName;
                    textReport.RegisterSession(CreateVcWssSession);
                }
                Assert.IsTrue(File.Exists(fileName));
            }
            finally
            {
                File.Delete(fileName);
            }
        }

        #endregion TextAdaptorTest

        #region MigrationEventArgs tests on TextAdaptor

        [TestMethod]
        public void SpecialXmlCharactersInDescriptionTest()
        {
            string fileName = TestUtils.TextReportRoot + "SpecialXmlCharactersInDescriptionTest.txt"; //bypass the filepath check
            string time;

            using (VersionControlSession sesssion = new VersionControlSession())
            using (TextReportAdaptor textReport = new TextReportAdaptor())
            {
                textReport.File = fileName;

                textReport.RegisterSession(sesssion);

                MigrationSessionEventArgs args = new MigrationSessionEventArgs(TestUtils.SpecialXMLCharacters);

                time = DateTime.Now.ToString();
                sesssion.OnSessionStart(args);

            }

            String firedEvent = readOutputLine(fileName);

            Regex exp = new Regex(String.Format(@"\[\d+, \d+, {0}, VC Session: {1}] {2}", time, "", TestUtils.SpecialXMLCharacters));
            // Check that proper output was given
            Assert.IsTrue(exp.IsMatch(firedEvent), String.Format("{0} did not match {1}", firedEvent, exp.ToString()));
        }

        [TestMethod]
        public void InvalidXmlCharactersInDescriptionTest()
        {
            string fileName = TestUtils.TextReportRoot + "InvalidXmlCharactersInDescriptionTest.txt"; //bypass the filepath check
            string time;
            using (VersionControlSession sesssion = new VersionControlSession())
            using (TextReportAdaptor textReport = new TextReportAdaptor())
            {
                textReport.File = fileName;

                textReport.RegisterSession(sesssion);

                MigrationSessionEventArgs args = new MigrationSessionEventArgs(TestUtils.InvalidXMLCharacters);

                time = DateTime.Now.ToString();
                sesssion.OnSessionStart(args);

            }

            String firedEvent = readOutputLine(fileName);

            Regex exp = new Regex(String.Format(@"\[\d+, \d+, {0}, VC Session: {1}] {2}", time, "", TestUtils.InvalidXMLCharacters));
            // Check that proper output was given
            Assert.IsTrue(exp.IsMatch(firedEvent), String.Format("{0} did not match {1}", firedEvent, exp.ToString()));
        }

        [TestMethod]
        public void NullDescriptionTest()
        {
            string time;
            string fileName = TestUtils.TextReportRoot + "NullDescriptionTest.txt"; //bypass the filepath check
            using (VersionControlSession sesssion = new VersionControlSession())
            using (TextReportAdaptor textReport = new TextReportAdaptor())
            {
                textReport.File = fileName;

                textReport.RegisterSession(sesssion);

                MigrationSessionEventArgs args = new MigrationSessionEventArgs(null);

                time = DateTime.Now.ToString();
                sesssion.OnSessionStart(args);

            }

            String firedEvent = readOutputLine(fileName);
            Regex exp = new Regex(String.Format(@"\[\d+, \d+, {0}, VC Session: {1}] {2}", time, "", null));

            // Check that proper output was given
            Assert.IsTrue(exp.IsMatch(firedEvent), String.Format("{0} did not match {1}", firedEvent, exp.ToString()));

        }
        #endregion MigrationEventArgs tests on TextAdaptor
    }
}
