﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.TeamFoundation.Migration.Toolkit;
using Microsoft.TeamFoundation.Migration.Toolkit.VC;
using System.IO;
using Tests.Framework;
using System.Data.SqlClient;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.Toolkit.VC.HighWaterMark&lt;T&gt; and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.Toolkit.VC.HighWaterMark&lt;T&gt; Unit Tests
    ///</summary>
    [TestClass()]
    public class HighWaterMarkTests : MigrationTestCaseBase
    {


        #region Additional test attributes
        [TestInitialize()]
        public override void Initialize()
        {
            base.Initialize();

            ConfigFile.WriteAndLoad(ConfigParameters);
            DataAccessManager.Current.CreateSchema(true);
        }

        [TestCleanup]
        public override void Cleanup()
        {
            base.Cleanup();

            string serverName = TestEnvironment.SqlServer;
            SqlUtil.DropDB(serverName, ConfigParameters.SqlInfo.Database);
        }
        #endregion


        /// <summary>
        ///A test for HighWaterMark
        ///</summary>
        [TestMethod()]
        public void HighWaterMarkTest()
        {
            VersionControlSession session = new VersionControlSession();
            session.Id = Guid.NewGuid().ToString();

            //testIntBaseHwm(session);
            testIntHwm(session);
        }

        bool wasDtMethodCalled;
        int expectedCurrent;
        int expectedNew;

        private void testIntHwm(VersionControlSession session)
        {
            // load the default hwm - since it's not found it defaults to default(DateTime)
            HighWaterMark<int> m_hwm = new HighWaterMark<int>(session, "TestIntHWM");
            expectedCurrent = default(int);

            Assert.AreEqual<int>(m_hwm.Value, expectedCurrent);

            m_hwm.BeforeUpdate += m_hwm_int_BeforeUpdate;

            // create a new DT and update.
            expectedNew = 15;
            m_hwm.Update(expectedNew);

            // make sure the callback was called
            Assert.IsTrue(wasDtMethodCalled);
            wasDtMethodCalled = false;

            // and that it did update to the proper value
            Assert.AreEqual<int>(expectedNew, m_hwm.Value);

            // load a new instance of the same hwm value
            HighWaterMark<int> m_hwm2 = new HighWaterMark<int>(session, "TestIntHWM");

            // it should have the same values as the previous instance
            assertDtAreSame(m_hwm.Value, m_hwm2.Value);

            // update the new instance
            expectedCurrent = expectedNew;
            expectedNew = 12345;
            m_hwm2.Update(expectedNew);

            wasDtMethodCalled = false;

            assertDtAreSame(expectedNew, m_hwm2.Value);

            // create a third instance and check it.
            HighWaterMark<int> m_hwm3 = new HighWaterMark<int>(session, "TestIntHWM");

            assertDtAreSame(m_hwm2.Value, m_hwm3.Value);

            // finally reload the origional and make sure it matches the third
            m_hwm.Reload();

            assertDtAreSame(m_hwm.Value, m_hwm3.Value);
            Assert.IsFalse(wasDtMethodCalled);  // reload should not update
        }

        void assertDtAreSame(int lhs, int rhs)
        {
            Assert.AreEqual<int>(lhs, rhs);
        }

        void m_hwm_int_BeforeUpdate(object sender, HighWaterMarkUpdatedEventArgs<int> args)
        {
            int currentHwm = args.Current;
            int newHwm = args.New;

            Assert.IsFalse(wasDtMethodCalled);

            wasDtMethodCalled = true;

            Assert.AreEqual<int>(expectedCurrent, currentHwm);
            Assert.AreEqual<int>(expectedNew, newHwm);
        }

        private void testIntBaseHwm(VersionControlSession session)
        {
            throw new NotImplementedException();
        }

        ///<summary>
        ///Scenario: Create a HighWaterMark with a null name; Create a HighWaterMark with an empty name
        ///Expected Result: ArgumentNullException 
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Create a HighWaterMark with a null name; Create a HighWaterMark with an empty name")]
        public void InvalidNameTest()
        {
            HighWaterMark<IConvertible> highWaterMark;
            try
            {
                highWaterMark = new HighWaterMark<IConvertible>(CreateVcWssSession, null);
                Assert.Fail("ExpectedException ArgumentNullException");
            }
            catch (ArgumentNullException) { }
        }

        ///<summary>
        ///Scenario: Create a HighWatermark with a null session
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Create a HighWatermark with a null session")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullSessionTest()
        {
            HighWaterMark<IConvertible> highWaterMark = new HighWaterMark<IConvertible>(null, "Name");
        }

        ///<summary>
        ///Scenario: Reload the HighWaterMark from the session variables
        ///Expected Result: Value stored in session variables becomes HighWaterMark.Value
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Reload the HighWaterMark from the session variables")]
        public void ReloadSessionVariableTest()
        {
            HighWaterMark<string> highWaterMark = new HighWaterMark<string>(CreateVcWssSession, "HighWaterMark");
            SessionVariables.SaveSessionVariable(CreateVcWssSession.Id, "HighWaterMark", "Reloaded Value");
            highWaterMark.Reload();
            Assert.AreEqual("Reloaded Value", highWaterMark.Value, "The high watermark was not loaded from session variables");
        }

        ///<summary>
        ///Scenario: Reload the HighWaterMark from the configuration file
        ///Expected Result: Value stored in the configuration file becomes HighWaterMark.Value
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Reload the HighWaterMark from the configuration file")]
        public void RelaodConfigSettingTest()
        {
            ConfigParameters.VersionControl.Sessions[0].Settings.Add("HighWaterMark", "Reloaded Value");

            HighWaterMark<string> highWaterMark = new HighWaterMark<string>(CreateVcWssSession, "HighWaterMark");

            highWaterMark.Reload();
            Assert.AreEqual("Reloaded Value", highWaterMark.Value, "The high watermark was not loaded from config file");
        }

        ///<summary>
        ///Scenario: Call update with a null NewValue
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call update with a null NewValue")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullNewValueUpdateTest()
        {
            HighWaterMark<IConvertible> highWaterMark = new HighWaterMark<IConvertible>(CreateVcWssSession, "Name");
            highWaterMark.Update(null);
        }

        ///<summary>
        ///Scenario: Call update with a null Sql Connection 
        ///Expected Result: ArgumentNullException
        ///</summary>
        [TestMethod(), Priority(3), Owner("curtisp")]
        [Description("Call update with a null Sql Connection ")]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullConnUpdateTest()
        {
            HighWaterMark<string> highWaterMark = new HighWaterMark<string>(CreateVcWssSession, "Name");
            highWaterMark.Update("later", null);
        }

        ///<summary>
        ///Scenario: Call update with valid parameters
        ///Expected Result: BeforeUpdate Event is fired. 
        ///</summary>
        [TestMethod(), Priority(1), Owner("curtisp")]
        [Description("Call update with valid parameters")]
        public void BeforeUpdateTest()
        {
            HighWaterMark<string> highWaterMark = new HighWaterMark<string>(CreateVcWssSession, "HighWaterMark");

            TestVCEventListeners listeners = new TestVCEventListeners();
            highWaterMark.BeforeUpdate += listeners.OnBeforeUpdate;

            highWaterMark.Update("Valid value");

            Assert.IsTrue(listeners.beforeUpdate, "On Before update was not fired");
        }
    }
}
