﻿// Copyright (c) Microsoft Corporation. All rights reserved.
// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using System.Diagnostics;
using Microsoft.TeamFoundation.Migration.Toolkit;
using Tests.Framework;

namespace Tests
{
    /// <summary>
    ///This is a test class for Microsoft.TeamFoundation.Migration.VersionControl.TraceManager and is intended
    ///to contain all Microsoft.TeamFoundation.Migration.VersionControl.TraceManager Unit Tests
    ///</summary>
    [TestClass()]
    public class TraceManagerTest : MigrationTestCaseBase
    {

        #region Additional test attributes

        [TestInitialize]
        public override void Initialize() 
        {
            base.Initialize();
            ConfigFile.WriteAndLoad(ConfigParameters);
            TraceManager.WarningsAsErrors = false;
        }
        #endregion


        /// <summary>
        ///A test for SwitchNotTest
        ///</summary>
        [TestMethod()]
        public void SwitchNotTest()
        {
            Assert.IsNotNull(TraceManager.Engine);
            Assert.IsNotNull(TraceManager.Toolkit);
        }

        /// <summary>
        ///A test for EnterMethod ()
        ///</summary>
        [TestMethod()]
        public void EnterMethodTest()
        {
            bool prevValue = TraceManager.ShowMethodDetails;

            try
            {
                TraceManager.ShowMethodDetails = true;
                TraceManager.EnterMethod();
                TraceManager.ExitMethod();

                TraceManager.EnterMethod(null);
                TraceManager.ExitMethod(null);

                object[] args = new object[]{
                    "this is arg 1",
                    "this is arg 2",
                    null,
                    10,
                    212390832092842412.298309,
                    DateTime.Now,
                    new byte[]{1, 2, 3, 4 }
                };

                TraceManager.EnterMethod(args);
                TraceManager.ExitMethod();

                EnterMethodTest_TestMethod(null, 12345, DateTime.Now, new object(), null);

                int intResult;
                EnterMethodTest_TestMethod_Out("string 1", 12345, DateTime.MinValue,
                    null, 'a', out intResult);

                EnterMethodTest_TestMethod_Params(
                    "{0} {1} {2}", "string 1", 0xDEADBEEF, DateTime.Now);
            }
            finally
            {
                TraceManager.ShowMethodDetails = prevValue;
            }
        }

        private DateTime EnterMethodTest_TestMethod(
            string arg1,
            int arg2,
            DateTime arg3,
            object arg4,
            string arg5)
        {
            TraceManager.EnterMethod(arg1, arg2, arg3, arg4, arg5);
            TraceManager.ExitMethod(arg3);
            return arg3;
        }

        private void EnterMethodTest_TestMethod_Out(
            string arg1,
            int arg2,
            DateTime arg3,
            object arg4,
            char arg5,
            out int result)
        {
            TraceManager.EnterMethod(arg1, arg2, arg3, arg4, arg5);
            result = 10;
            TraceManager.ExitMethod(result);
        }

        private void EnterMethodTest_TestMethod_Params(
            string format,
            params object[] args)
        {
            TraceManager.EnterMethod(format, args);
            TraceManager.ExitMethod();
        }


        /// <summary>
        ///A test for StartLogicalOperation (object)
        ///</summary>
        [TestMethod()]
        public void StartLogicalOperationTest()
        {
            string expected = "StartLogicalOperationTest_TEST1";
            using (TraceManager.StartLogicalOperation(expected))
            {
                string actual = (string)Trace.CorrelationManager.LogicalOperationStack.Peek();
                Assert.AreEqual<string>(expected, actual);

                int count = Trace.CorrelationManager.LogicalOperationStack.Count;
                Assert.IsTrue(count > 0);
                Assert.AreNotEqual<int>(count, 0);

                string expected2 = "StartLogicalOperationTest_TEST2";
                using (TraceManager.StartLogicalOperation(expected2))
                {
                    string actual2 = (string)Trace.CorrelationManager.LogicalOperationStack.Peek();
                    Assert.AreEqual<string>(expected2, actual2);

                    int count2 = Trace.CorrelationManager.LogicalOperationStack.Count;
                    Assert.IsTrue(count2 > 0);
                    Assert.AreNotEqual<int>(count2, 0);
                }
            }
        }

        /// <summary>
        ///A test for TraceError (string)
        ///</summary>
        [TestMethod()]
        public void TraceErrorTest()
        {
            bool prevValue = TraceManager.AbortOnError;

            if (prevValue)
            {
                TraceManager.AbortOnError = false;
            }

            TraceManager.TraceError(null);
            TraceManager.TraceError(string.Empty);
            TraceManager.TraceError("This is the error string");
            TraceManager.TraceError("Formatted string: {0}", 12345);

            TraceManager.AbortOnError = true;

            try
            {
                TraceManager.TraceError("should throw exception");
                Assert.Fail("Should have thrown an exception on error");
            }
            catch (MigrationException)
            {
            }

            TraceManager.AbortOnError = prevValue;
        }

        /// <summary>
        ///A test for TraceInformation (string)
        ///</summary>
        [TestMethod()]
        public void TraceInformationTest()
        {
            TraceManager.TraceInformation(null);
            TraceManager.TraceInformation(string.Empty);
            TraceManager.TraceInformation("This is the error string");
            TraceManager.TraceInformation("Formatted string: {0}", 12345);
        }

        /// <summary>
        ///A test for TraceWarning (string)
        ///</summary>
        [TestMethod()]
        public void TraceWarningTest()
        {
            TraceManager.TraceWarning(null);
            TraceManager.TraceWarning(string.Empty);
            TraceManager.TraceWarning("This is the error string");
            TraceManager.TraceWarning("Formatted string: {0}", 12345);
        }

        /// <summary>
        ///A test for WriteLine (string)
        ///</summary>
        [DeploymentItem("Microsoft.TeamFoundation.Migration.VersionControl.dll")]
        [TestMethod()]
        public void WriteLineTest()
        {
            TraceManager.WriteLine(null, "null switch");
            TraceManager.WriteLine(null, null);
            TraceManager.WriteLine(null, string.Empty);

            TraceLevel[] levels = new TraceLevel[]
            {
                TraceLevel.Error,
                TraceLevel.Info,
                TraceLevel.Off,
                TraceLevel.Verbose,
                TraceLevel.Warning
            };

            foreach (TraceLevel level in levels)
            {
                TraceSwitch sw = new TraceSwitch("test switch", "description");
                sw.Level = level;

                TraceManager.WriteLine(sw, null);
                TraceManager.WriteLine(sw, string.Empty);
                TraceManager.WriteLine(sw, "_data");
            }
        }

        /// <summary>
        ///A test for WriteLine (TraceSwitch, string, params object[])
        ///</summary>
        [TestMethod()]
        public void WriteLineTest1()
        {
            TraceManager.WriteLine(TraceManager.Engine, "{0}:{1}", "argument one", 10);

            try
            {
                TraceManager.WriteLine(null, "{0}:{1}", "argument one", 10);
                Assert.Fail("null TraceSwitch argument should have thrown exception");
            }
            catch (ArgumentNullException)
            {
            }

            try
            {
                TraceManager.WriteLine(TraceManager.Engine, null, "argument one", 10);
                Assert.Fail("null format argument should have thrown exception");
            }
            catch (ArgumentNullException)
            {
            }
        }

        /// <summary>
        ///A test for TraceError (Exception, bool)
        ///</summary>
        [TestMethod()]
        public void TraceTest()
        {
            TraceManager.AbortOnError = false;
            TraceManager.WarningsAsErrors = false;

            TraceManager.TraceWarning("should not throw");
            TraceManager.TraceError("message");
            TraceManager.TraceError(new Exception(), false);

            TraceManager.WarningsAsErrors = true;

            TraceManager.TraceWarning("should not throw");
            TraceManager.TraceError("message");
            TraceManager.TraceError(new Exception(), false);

            TraceManager.AbortOnError = true;

            demandException(true, "{0}", "message");
            demandException(false, "{0}", "message");
            demandException(new Exception(), false);
            demandException(new Exception(), true);
        }

        private void demandException(bool callWarn, string format, params object[] args)
        {
            try
            {
                if (callWarn)
                {
                    TraceManager.TraceWarning(format, args);
                }
                else
                {
                    TraceManager.TraceError(format, args);
                }
                Assert.Fail("Should have thrown migration exception");
            }
            catch (MigrationException)
            {
            }
        }

        private void demandException(Exception e, bool forceThrow)
        {
            try
            {
                TraceManager.TraceError(e, forceThrow);
                Assert.Fail("Should have thrown migration exception");
            }
            catch (MigrationException)
            {
            }
        }
    }
}
